@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,11 +31,13 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include <iomanip>
 #include <set>  // std::set
 #include <sstream>
+#include <unordered_set>
 
 #include "mysql/components/library_mysys/my_memory.h"
 #include "mysql/components/services/mysql_rwlock.h"
 #include "mysql/components/services/psi_memory.h"
 #include "mysqld_error.h"
+#include "scope_guard.h"
 
 #define PSI_NOT_INSTRUMENTED 0
 const int MAX_DICTIONARY_FILE_LENGTH = (1024 * 1024);
@@ -79,7 +82,7 @@ static TYPE_LIB password_policy_typelib_t = {array_elements(policy_names) - 1,
 
 typedef std::string string_type;
 typedef std::set<string_type> set_type;
-static set_type dictionary_words;
+static set_type *dictionary_words{nullptr};
 
 static int validate_password_length;
 static int validate_password_number_count;
@@ -90,6 +93,7 @@ static char *validate_password_dictionary_file;
 static char *validate_password_dictionary_file_last_parsed = nullptr;
 static long long validate_password_dictionary_file_words_count = 0;
 static bool check_user_name;
+static int validate_password_changed_characters_percentage = 0;
 /*
   This variable is used, to make sure the use of component services
   after the component load/initialization is done.
@@ -131,8 +135,8 @@ static void dictionary_activate(set_type *dict_words) {
      << std::setfill('0') << std::setw(2) << tm.tm_sec;
 
   mysql_rwlock_wrlock(&LOCK_dict_file);
-  std::swap(dictionary_words, *dict_words);
-  validate_password_dictionary_file_words_count = dictionary_words.size();
+  std::swap(*dictionary_words, *dict_words);
+  validate_password_dictionary_file_words_count = dictionary_words->size();
   /*
     We are re-using 'validate_password_dictionary_file_last_parsed'
     so, we need to make sure to free the previously allocated memory.
@@ -204,7 +208,7 @@ static void read_dictionary_file() {
 /* Clear words from std::set */
 static void free_dictionary_file() {
   mysql_rwlock_wrlock(&LOCK_dict_file);
-  if (!dictionary_words.empty()) dictionary_words.clear();
+  if (!dictionary_words->empty()) dictionary_words->clear();
   if (validate_password_dictionary_file_last_parsed) {
     my_free(validate_password_dictionary_file_last_parsed);
     validate_password_dictionary_file_last_parsed = nullptr;
@@ -221,7 +225,7 @@ static int validate_dictionary_check(my_h_string password) {
   char *buffer;
   my_h_string lower_string_handle;
 
-  if (dictionary_words.empty()) return (1);
+  if (dictionary_words->empty()) return (1);
 
   /* New String is allocated */
   if (mysql_service_mysql_string_factory->create(&lower_string_handle)) {
@@ -244,7 +248,7 @@ static int validate_dictionary_check(my_h_string password) {
     return (0);
 
   if (mysql_service_mysql_string_converter->convert_to_buffer(
-          lower_string_handle, buffer, MAX_PASSWORD_LENGTH, "utf8")) {
+          lower_string_handle, buffer, MAX_PASSWORD_LENGTH, "utf8mb3")) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(ERROR_LEVEL)
@@ -268,8 +272,8 @@ static int validate_dictionary_check(my_h_string password) {
     substr_pos = 0;
     while (substr_pos + substr_length <= length) {
       password_substr = password_str.substr(substr_pos, substr_length);
-      itr = dictionary_words.find(password_substr);
-      if (itr != dictionary_words.end()) {
+      itr = dictionary_words->find(password_substr);
+      if (itr != dictionary_words->end()) {
         mysql_rwlock_unlock(&LOCK_dict_file);
         my_free(buffer);
         return (0);
@@ -374,7 +378,7 @@ static bool is_valid_password_by_user_name(void *thd, my_h_string password) {
   }
 
   if (mysql_service_mysql_string_converter->convert_to_buffer(
-          password, buffer, MAX_PASSWORD_LENGTH, "utf8")) {
+          password, buffer, MAX_PASSWORD_LENGTH, "utf8mb3")) {
     LogEvent()
         .type(LOG_TYPE_ERROR)
         .prio(WARNING_LEVEL)
@@ -481,7 +485,7 @@ static int validate_password_policy_strength(void *thd, my_h_string password,
     return (0);
   }
   while (mysql_service_mysql_string_iterator->iterator_get_next(
-             iter, &out_iter_char) != true) {
+             iter, &out_iter_char) == 0) {
     n_chars++;
     if (policy > PASSWORD_POLICY_LOW) {
       if (!mysql_service_mysql_string_ctype->is_lower(iter, &out) && out)
@@ -551,7 +555,7 @@ DEFINE_BOOL_METHOD(validate_password_imp::get_strength,
     return true;
   }
   while (mysql_service_mysql_string_iterator->iterator_get_next(
-             iter, &out_iter_char) != true)
+             iter, &out_iter_char) == 0)
     n_chars++;
 
   mysql_service_mysql_string_iterator->iterator_destroy(iter);
@@ -594,6 +598,114 @@ DEFINE_BOOL_METHOD(validate_password_imp::validate,
                                             validate_password_policy) == 0);
 }
 
+/**
+  Validate if number of changed characters matches the pre-configured
+  criteria
+
+  @param [in]  current_password Current password
+  @param [in]  new_password     New password
+  @param [out] minimum_required Minimum required number of changed characters
+  @param [out] changed          Actual number of changed characters
+
+  @returns Result of validation
+    @retval false Success
+    @retval true  Error
+*/
+DEFINE_BOOL_METHOD(validate_password_changed_characters_imp::validate,
+                   (my_h_string current_password, my_h_string new_password,
+                    uint *minimum_required, uint *changed)) {
+  try {
+    uint current_length = 0, new_length = 0;
+    if (changed) *changed = 0;
+
+    /* quick exit if restriction is not imposed */
+    if (validate_password_changed_characters_percentage == 0) return false;
+
+    /* Convert passwords to lowercase before comparison */
+    my_h_string current_password_lc, new_password_lc;
+    if (mysql_service_mysql_string_factory->create(&current_password_lc) ||
+        mysql_service_mysql_string_factory->create(&new_password_lc)) {
+      LogEvent()
+          .type(LOG_TYPE_ERROR)
+          .prio(ERROR_LEVEL)
+          .lookup(ER_VALIDATE_PWD_STRING_HANDLER_MEM_ALLOCATION_FAILED);
+      return true;
+    }
+
+    auto cleanup_guard = create_scope_guard([&] {
+      mysql_service_mysql_string_factory->destroy(current_password_lc);
+      mysql_service_mysql_string_factory->destroy(new_password_lc);
+    });
+
+    if (mysql_service_mysql_string_case->tolower(&current_password_lc,
+                                                 current_password) ||
+        mysql_service_mysql_string_case->tolower(&new_password_lc,
+                                                 new_password)) {
+      LogEvent()
+          .type(LOG_TYPE_ERROR)
+          .prio(ERROR_LEVEL)
+          .lookup(ER_VALIDATE_PWD_STRING_CONV_TO_LOWERCASE_FAILED);
+      return true;
+    }
+
+    if (mysql_service_mysql_string_character_access->get_char_length(
+            current_password_lc, &current_length) ||
+        mysql_service_mysql_string_character_access->get_char_length(
+            new_password_lc, &new_length)) {
+      return true;
+    }
+
+    /* Determine number of characters required to be changed */
+    uint number_of_characters_to_be_changed =
+        (std::max(static_cast<uint>(validate_password_length), current_length) *
+         (static_cast<uint>(validate_password_changed_characters_percentage)) /
+         100);
+
+    if (minimum_required)
+      *minimum_required = number_of_characters_to_be_changed;
+
+    std::unordered_set<long> characters;
+    auto process_password = [&characters](my_h_string password,
+                                          bool add) -> bool {
+      int pos = 0;
+      ulong character = 0;
+      my_h_string_iterator password_iterator{nullptr};
+
+      if (mysql_service_mysql_string_iterator->iterator_create(
+              password, &password_iterator))
+        return true;
+
+      auto iterator_cleanup = create_scope_guard([&] {
+        mysql_service_mysql_string_iterator->iterator_destroy(
+            password_iterator);
+      });
+
+      while (!mysql_service_mysql_string_iterator->iterator_get_next(
+          password_iterator, &pos)) {
+        if (mysql_service_mysql_string_value->get(password_iterator,
+                                                  &character))
+          return true;
+
+        if (add)
+          (void)characters.insert(character);
+        else
+          (void)characters.erase(character);
+      }
+      return false;
+    };
+
+    if (process_password(new_password_lc, true)) return true;
+
+    if (process_password(current_password_lc, false)) return true;
+
+    if (changed) *changed = characters.size();
+
+    return (characters.size() < number_of_characters_to_be_changed);
+  } catch (...) {
+    return true;
+  }
+}
+
 int register_status_variables() {
   if (mysql_service_status_variable_registration->register_variable(
           (SHOW_VAR *)&validate_password_status_variables)) {
@@ -607,7 +719,10 @@ int register_status_variables() {
 }
 
 int register_system_variables() {
-  INTEGRAL_CHECK_ARG(int) length, num_count, mixed_case_count, spl_char_count;
+  INTEGRAL_CHECK_ARG(int)
+  length, num_count, mixed_case_count, spl_char_count,
+      changed_characters_percentage;
+
   length.def_val = 8;
   length.min_val = 0;
   length.max_val = 0;
@@ -731,7 +846,30 @@ int register_system_variables() {
                 "validate_password.check_user_name");
     goto check_user_name;
   }
+
+  changed_characters_percentage.def_val = 0;
+  changed_characters_percentage.min_val = 0;
+  changed_characters_percentage.max_val = 100;
+  changed_characters_percentage.blk_sz = 0;
+  if (mysql_service_component_sys_variable_register->register_variable(
+          "validate_password", "changed_characters_percentage",
+          PLUGIN_VAR_INT | PLUGIN_VAR_RQCMDARG,
+          "password validate percentage of changed characters required in new "
+          "password. Valid values between 0 and 100.",
+          nullptr, length_update, (void *)&changed_characters_percentage,
+          (void *)&validate_password_changed_characters_percentage)) {
+    LogEvent()
+        .type(LOG_TYPE_ERROR)
+        .prio(ERROR_LEVEL)
+        .lookup(ER_VALIDATE_PWD_VARIABLE_REGISTRATION_FAILED,
+                "validate_password.changed_characters_percentage");
+    goto changed_characters_percentage;
+  }
   return 0; /* All system variables registered successfully */
+
+changed_characters_percentage:
+  mysql_service_component_sys_variable_unregister->unregister_variable(
+      "validate_password", "check_user_name");
 check_user_name:
   mysql_service_component_sys_variable_unregister->unregister_variable(
       "validate_password", "dictionary_file");
@@ -828,6 +966,15 @@ int unregister_system_variables() {
         .lookup(ER_VALIDATE_PWD_VARIABLE_UNREGISTRATION_FAILED,
                 "validate_password.check_user_name");
   }
+
+  if (mysql_service_component_sys_variable_unregister->unregister_variable(
+          "validate_password", "changed_characters_percentage")) {
+    LogEvent()
+        .type(LOG_TYPE_ERROR)
+        .prio(ERROR_LEVEL)
+        .lookup(ER_VALIDATE_PWD_VARIABLE_UNREGISTRATION_FAILED,
+                "validate_password.changed_characters_percentage");
+  }
   return 0;
 }
 
@@ -867,11 +1014,30 @@ bool log_service_deinit() { return false; }
   @retval true failure
 */
 static mysql_service_status_t validate_password_init() {
+  dictionary_words = new set_type();
   init_validate_password_psi_keys();
   mysql_rwlock_init(key_validate_password_LOCK_dict_file, &LOCK_dict_file);
-  if (log_service_init() || register_system_variables() ||
-      register_status_variables())
+  if (log_service_init()) {
+    delete dictionary_words;
+    dictionary_words = nullptr;
+    mysql_rwlock_destroy(&LOCK_dict_file);
     return true;
+  }
+  if (register_system_variables()) {
+    log_service_deinit();
+    delete dictionary_words;
+    dictionary_words = nullptr;
+    mysql_rwlock_destroy(&LOCK_dict_file);
+    return true;
+  }
+  if (register_status_variables()) {
+    unregister_system_variables();
+    log_service_deinit();
+    delete dictionary_words;
+    dictionary_words = nullptr;
+    mysql_rwlock_destroy(&LOCK_dict_file);
+    return true;
+  }
   read_dictionary_file();
   /* Check if validate_password_length needs readjustment */
   readjust_validate_password_length();
@@ -889,6 +1055,8 @@ static mysql_service_status_t validate_password_init() {
 static mysql_service_status_t validate_password_deinit() {
   free_dictionary_file();
   mysql_rwlock_destroy(&LOCK_dict_file);
+  delete dictionary_words;
+  dictionary_words = nullptr;
   if (unregister_system_variables() || unregister_status_variables() ||
       log_service_deinit())
     return true;
@@ -900,9 +1068,14 @@ BEGIN_SERVICE_IMPLEMENTATION(validate_password, validate_password)
 validate_password_imp::validate,
     validate_password_imp::get_strength END_SERVICE_IMPLEMENTATION();
 
+BEGIN_SERVICE_IMPLEMENTATION(validate_password,
+                             validate_password_changed_characters)
+validate_password_changed_characters_imp::validate END_SERVICE_IMPLEMENTATION();
+
 /* component provides: the validate_password service */
 BEGIN_COMPONENT_PROVIDES(validate_password)
 PROVIDES_SERVICE(validate_password, validate_password),
+    PROVIDES_SERVICE(validate_password, validate_password_changed_characters),
     END_COMPONENT_PROVIDES();
 
 /* A block for specifying dependencies of this Component. Note that for each
@@ -910,11 +1083,13 @@ PROVIDES_SERVICE(validate_password, validate_password),
   header file of the Component, and an entry on requires list below. */
 REQUIRES_SERVICE_PLACEHOLDER(log_builtins);
 REQUIRES_SERVICE_PLACEHOLDER(log_builtins_string);
+REQUIRES_SERVICE_PLACEHOLDER(mysql_string_character_access);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_factory);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_case);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_converter);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_iterator);
 REQUIRES_SERVICE_PLACEHOLDER(mysql_string_ctype);
+REQUIRES_SERVICE_PLACEHOLDER(mysql_string_value);
 REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_register);
 REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_unregister);
 REQUIRES_SERVICE_PLACEHOLDER(status_variable_registration);
@@ -928,12 +1103,12 @@ REQUIRES_MYSQL_RWLOCK_SERVICE_PLACEHOLDER;
    component load time and disposes off them at unload.
 */
 BEGIN_COMPONENT_REQUIRES(validate_password)
-REQUIRES_SERVICE(registry), REQUIRES_SERVICE(log_builtins),
-    REQUIRES_SERVICE(log_builtins_string),
+REQUIRES_SERVICE(log_builtins), REQUIRES_SERVICE(log_builtins_string),
+    REQUIRES_SERVICE(mysql_string_character_access),
     REQUIRES_SERVICE(mysql_string_factory), REQUIRES_SERVICE(mysql_string_case),
     REQUIRES_SERVICE(mysql_string_converter),
     REQUIRES_SERVICE(mysql_string_iterator),
-    REQUIRES_SERVICE(mysql_string_ctype),
+    REQUIRES_SERVICE(mysql_string_ctype), REQUIRES_SERVICE(mysql_string_value),
     REQUIRES_SERVICE(component_sys_variable_register),
     REQUIRES_SERVICE(component_sys_variable_unregister),
     REQUIRES_SERVICE(status_variable_registration),
