@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -494,8 +495,8 @@ static inline void inline_mysql_file_register(
 #ifdef HAVE_PSI_FILE_INTERFACE
     const char *category, PSI_file_info *info, int count
 #else
-    const char *category MY_ATTRIBUTE((unused)),
-    void *info MY_ATTRIBUTE((unused)), int count MY_ATTRIBUTE((unused))
+    const char *category [[maybe_unused]], void *info [[maybe_unused]],
+    int count [[maybe_unused]]
 #endif
 ) {
 #ifdef HAVE_PSI_FILE_INTERFACE
