@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -28,15 +29,18 @@
 #include <mysql/components/services/log_builtins.h>
 #include "my_dbug.h"
 #include "plugin/group_replication/include/plugin.h"
+#include "plugin/group_replication/include/plugin_messages/transaction_message.h"
 
 const std::string Gcs_operations::gcs_engine = "xcom";
 
 Gcs_operations::Gcs_operations()
-    : gcs_interface(nullptr),
+    : auth_provider(),
+      native_interface(),
+      gcs_mysql_net_provider(nullptr),
+      gcs_interface(nullptr),
       injected_view_modification(false),
       leave_coordination_leaving(false),
-      leave_coordination_left(false),
-      finalize_ongoing(false) {
+      leave_coordination_left(false) {
   gcs_operations_lock = new Checkable_rwlock(
 #ifdef HAVE_PSI_INTERFACE
       key_GR_RWLOCK_gcs_operations
@@ -45,11 +49,6 @@ Gcs_operations::Gcs_operations()
   view_observers_lock = new Checkable_rwlock(
 #ifdef HAVE_PSI_INTERFACE
       key_GR_RWLOCK_gcs_operations_view_change_observers
-#endif
-  );
-  finalize_ongoing_lock = new Checkable_rwlock(
-#ifdef HAVE_PSI_INTERFACE
-      key_GR_RWLOCK_gcs_operations_finalize_ongoing
 #endif
   );
 }
@@ -57,12 +56,12 @@ Gcs_operations::Gcs_operations()
 Gcs_operations::~Gcs_operations() {
   delete gcs_operations_lock;
   delete view_observers_lock;
-  delete finalize_ongoing_lock;
 }
 
 int Gcs_operations::initialize() {
   DBUG_TRACE;
   int error = 0;
+  Gcs_interface_runtime_requirements reqs;
 
   gcs_operations_lock->wrlock();
 
@@ -89,6 +88,17 @@ int Gcs_operations::initialize() {
     /* purecov: end */
   }
 
+  // Setup resources and pass them to GCS. In the future, refactor to also pass
+  // the logger.
+  if (gcs_mysql_net_provider ==
+      nullptr) {  // finalize might have not been called.
+    gcs_mysql_net_provider = std::make_shared<Gcs_mysql_network_provider>(
+        &auth_provider, &native_interface);
+  }
+
+  reqs.provider = gcs_mysql_net_provider;
+  reqs.namespace_manager = &native_interface;
+  gcs_interface->setup_runtime_resources(reqs);
 end:
   gcs_operations_lock->unlock();
   return error;
@@ -96,19 +106,24 @@ end:
 
 void Gcs_operations::finalize() {
   DBUG_TRACE;
-  finalize_ongoing_lock->wrlock();
-  finalize_ongoing = true;
   gcs_operations_lock->wrlock();
-  finalize_ongoing_lock->unlock();
 
   if (gcs_interface != nullptr) gcs_interface->finalize();
+
+  // Remove the provider since it is owned by the plugin.
+  if (gcs_interface != nullptr) {
+    Gcs_interface_runtime_requirements reqs;
+    reqs.provider = gcs_mysql_net_provider;
+    gcs_interface->cleanup_runtime_resources(reqs);
+  }
+
   Gcs_interface_factory::cleanup(gcs_engine);
   gcs_interface = nullptr;
 
-  finalize_ongoing_lock->wrlock();
-  finalize_ongoing = false;
+  // Will destroy gcs_mysql_net_provider if it is still alive after finalize
+  gcs_mysql_net_provider = nullptr;
+
   gcs_operations_lock->unlock();
-  finalize_ongoing_lock->unlock();
 }
 
 enum enum_gcs_error Gcs_operations::configure(
@@ -117,7 +132,16 @@ enum enum_gcs_error Gcs_operations::configure(
   enum enum_gcs_error error = GCS_NOK;
   gcs_operations_lock->wrlock();
 
-  if (gcs_interface != nullptr) error = gcs_interface->initialize(parameters);
+  if (gcs_interface != nullptr) {
+    error = gcs_interface->initialize(parameters);
+
+    // This forces GCS to create group interfaces under a RW lock.
+    if (gcs_interface->is_initialized()) {
+      std::string group_name(get_group_name_var());
+      Gcs_group_identifier group_id(group_name);
+      gcs_interface->get_communication_session(group_id);
+    }
+  }
 
   gcs_operations_lock->unlock();
   return error;
@@ -283,6 +307,12 @@ Gcs_operations::enum_leave_state Gcs_operations::leave(
       goto end;
       /* purecov: end */
     }
+  } else {  // This is the "something went wrong branch". We will avoid
+            // incoming connections to ensure that we do not end in a deadlock,
+            // thus de-registering the incoming connection callback
+    if (gcs_mysql_net_provider != nullptr) {
+      gcs_mysql_net_provider->stop();
+    }
   }
 
 end:
@@ -327,28 +357,8 @@ void Gcs_operations::remove_view_notifer(
 void Gcs_operations::leave_coordination_member_left() {
   DBUG_TRACE;
 
-  /*
-    If finalize method is ongoing, it means that GCS is waiting that
-    all messages and views are delivered to GR, if we proceed with
-    this method we will enter on the deadlock:
-      1) leave view was not delivered before wait view timeout;
-      2) finalize did start and acquired lock->wrlock();
-      3) leave view was delivered, member_left is waiting to
-         acquire lock->wrlock().
-    So, if leaving, we just do nothing.
-  */
-  finalize_ongoing_lock->rdlock();
-  if (finalize_ongoing) {
-    finalize_ongoing_lock->unlock();
-    return;
-  }
-  gcs_operations_lock->wrlock();
-  finalize_ongoing_lock->unlock();
-
   leave_coordination_leaving = false;
   leave_coordination_left = true;
-
-  gcs_operations_lock->unlock();
 }
 
 Gcs_view *Gcs_operations::get_current_view() {
@@ -436,15 +446,58 @@ enum enum_gcs_error Gcs_operations::send_message(
   return error;
 }
 
-int Gcs_operations::force_members(const char *members) {
+enum enum_gcs_error Gcs_operations::send_transaction_message(
+    Transaction_message_interface &message) {
   DBUG_TRACE;
-  int error = 0;
+  enum enum_gcs_error error = GCS_NOK;
+  gcs_operations_lock->rdlock();
+
+  /*
+    Ensure that group communication interfaces are initialized
+    and ready to use, since plugin can leave the group on errors
+    but continue to be active.
+  */
+  if (gcs_interface == nullptr || !gcs_interface->is_initialized()) {
+    gcs_operations_lock->unlock();
+    return GCS_NOK;
+  }
+
+  std::string group_name(get_group_name_var());
+  Gcs_group_identifier group_id(group_name);
+
+  Gcs_communication_interface *gcs_communication =
+      gcs_interface->get_communication_session(group_id);
+  Gcs_control_interface *gcs_control =
+      gcs_interface->get_control_session(group_id);
+
+  if (gcs_communication == nullptr || gcs_control == nullptr) {
+    gcs_operations_lock->unlock();
+    return GCS_NOK;
+  }
+
+  Gcs_member_identifier origin = gcs_control->get_local_member_identifier();
+  Gcs_message_data *gcs_message_data = message.get_message_data_and_reset();
+  if (nullptr == gcs_message_data) {
+    gcs_operations_lock->unlock();
+    return GCS_NOK;
+  }
+  Gcs_message gcs_message(origin, gcs_message_data);
+  error = gcs_communication->send_message(gcs_message);
+
+  gcs_operations_lock->unlock();
+  return error;
+}
+
+Gcs_operations::enum_force_members_state Gcs_operations::force_members(
+    const char *members) {
+  DBUG_TRACE;
+  enum_force_members_state error = FORCE_MEMBERS_OK;
   gcs_operations_lock->wrlock();
 
   if (gcs_interface == nullptr || !gcs_interface->is_initialized()) {
     /* purecov: begin inspected */
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_GRP_MEMBER_OFFLINE);
-    error = 1;
+    error = FORCE_MEMBERS_ER_MEMBER_NOT_ONLINE;
     goto end;
     /* purecov: end */
   }
@@ -455,7 +508,7 @@ int Gcs_operations::force_members(const char *members) {
   */
   if (leave_coordination_leaving) {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FORCE_MEMBERS_WHEN_LEAVING);
-    error = 1;
+    error = FORCE_MEMBERS_ER_MEMBERS_WHEN_LEAVING;
     goto end;
   }
 
@@ -469,7 +522,7 @@ int Gcs_operations::force_members(const char *members) {
     if (gcs_management == nullptr) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_GCS_INTERFACE_ERROR);
-      error = 1;
+      error = FORCE_MEMBERS_ER_INTERNAL_ERROR;
       goto end;
       /* purecov: end */
     }
@@ -490,24 +543,25 @@ int Gcs_operations::force_members(const char *members) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FORCE_MEMBER_VALUE_SET_ERROR,
                    members);
-      error = 1;
+      error = FORCE_MEMBERS_ER_VALUE_SET_ERROR;
       view_change_notifier.cancel_view_modification();
       remove_view_notifer(&view_change_notifier);
       goto end;
       /* purecov: end */
     }
     LogPluginErr(SYSTEM_LEVEL, ER_GRP_RPL_FORCE_MEMBER_VALUE_SET, members);
-    if (view_change_notifier.wait_for_view_modification()) {
+    if (view_change_notifier.wait_for_view_modification(
+            FORCE_MEMBERS_VIEW_MODIFICATION_TIMEOUT)) {
       /* purecov: begin inspected */
       LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_FORCE_MEMBER_VALUE_TIME_OUT,
                    members);
-      error = 1;
+      error = FORCE_MEMBERS_ER_TIMEOUT_ON_WAIT_FOR_VIEW;
       /* purecov: end */
     }
     remove_view_notifer(&view_change_notifier);
   } else {
     LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_GRP_MEMBER_OFFLINE);
-    error = 1;
+    error = FORCE_MEMBERS_ER_MEMBER_NOT_ONLINE;
     goto end;
   }
 
@@ -570,6 +624,48 @@ enum enum_gcs_error Gcs_operations::set_write_concurrency(
   return result;
 }
 
+enum enum_gcs_error Gcs_operations::set_leader(
+    Gcs_member_identifier const &leader) {
+  DBUG_TRACE;
+  enum enum_gcs_error result = GCS_NOK;
+  gcs_operations_lock->wrlock();
+  Gcs_group_management_interface *gcs_group_manager = get_gcs_group_manager();
+  if (gcs_group_manager != nullptr) {
+    result = gcs_group_manager->set_single_leader(leader);
+  }
+
+  gcs_operations_lock->unlock();
+  return result;
+}
+
+enum enum_gcs_error Gcs_operations::set_everyone_leader() {
+  DBUG_TRACE;
+  enum enum_gcs_error result = GCS_NOK;
+  gcs_operations_lock->wrlock();
+  Gcs_group_management_interface *gcs_group_manager = get_gcs_group_manager();
+  if (gcs_group_manager != nullptr) {
+    result = gcs_group_manager->set_everyone_leader();
+  }
+
+  gcs_operations_lock->unlock();
+  return result;
+}
+
+enum enum_gcs_error Gcs_operations::get_leaders(
+    std::vector<Gcs_member_identifier> &preferred_leaders,
+    std::vector<Gcs_member_identifier> &actual_leaders) {
+  DBUG_TRACE;
+  enum enum_gcs_error result = GCS_NOK;
+  gcs_operations_lock->rdlock();
+  Gcs_group_management_interface *gcs_group_manager = get_gcs_group_manager();
+  if (gcs_group_manager != nullptr) {
+    result = gcs_group_manager->get_leaders(preferred_leaders, actual_leaders);
+  }
+
+  gcs_operations_lock->unlock();
+  return result;
+}
+
 uint32_t Gcs_operations::get_minimum_write_concurrency() const {
   DBUG_TRACE;
   uint32_t result = 0;
@@ -681,6 +777,39 @@ enum enum_gcs_error Gcs_operations::set_xcom_cache_size(uint64_t new_size) {
   return result;
 }
 
+enum_transport_protocol
+Gcs_operations::get_current_incoming_connections_protocol() {
+  DBUG_TRACE;
+  enum_transport_protocol result = INVALID_PROTOCOL;
+  gcs_operations_lock->rdlock();
+  if (gcs_interface != nullptr && gcs_interface->is_initialized()) {
+    std::string group_name(get_group_name_var());
+    Gcs_group_identifier group_id(group_name);
+    Gcs_communication_interface *gcs_comms =
+        gcs_interface->get_communication_session(group_id);
+
+    if (gcs_comms != nullptr) {
+      result = gcs_comms->get_incoming_connections_protocol();
+    }
+  }
+  gcs_operations_lock->unlock();
+  return result;
+}
+
+Gcs_mysql_network_provider *Gcs_operations::get_mysql_network_provider() {
+  DBUG_TRACE;
+  Gcs_mysql_network_provider *result = nullptr;
+
+  gcs_operations_lock->rdlock();
+  if (gcs_interface != nullptr && gcs_mysql_net_provider != nullptr &&
+      gcs_interface->is_initialized()) {
+    result = gcs_mysql_net_provider.get();
+  }
+  gcs_operations_lock->unlock();
+
+  return result;
+}
+
 const std::string &Gcs_operations::get_gcs_engine() { return gcs_engine; }
 
 bool Gcs_operations::is_initialized() {
