@@ -1,14 +1,15 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -178,7 +179,7 @@ void Security_context::skip_grants(const char *user /*= "skip-grants user"*/,
   set_host_or_ip_ptr("", 0);
   assign_priv_user(user, strlen(user));
   assign_priv_host(host, strlen(host));
-  m_master_access = ~NO_ACCESS;
+  m_master_access = ALL_ACCESS;
   m_is_skip_grants_user = true;
 
   /*
@@ -316,7 +317,7 @@ bool Security_context::user_matches(Security_context *them) {
          !strcmp(m_user.ptr(), them_user);
 }
 
-bool Security_context::check_access(ulong want_access,
+bool Security_context::check_access(Access_bitmask want_access,
                                     const std::string &db_name /* = "" */,
                                     bool match_any) {
   DBUG_TRACE;
@@ -328,7 +329,8 @@ bool Security_context::check_access(ulong want_access,
                     : ((m_master_access & want_access) == want_access));
 }
 
-ulong Security_context::master_access(const std::string &db_name) const {
+Access_bitmask Security_context::master_access(
+    const std::string &db_name) const {
   return filter_access(m_master_access, db_name);
 }
 
@@ -406,7 +408,7 @@ void Security_context::checkout_access_maps(void) {
   if (m_acl_map != nullptr) {
     DBUG_PRINT("info",
                ("Roles are active and global access for %.*s@%.*s is set to"
-                " %lu",
+                " %" PRIu32,
                 (int)m_priv_user_length, m_priv_user, (int)m_priv_host_length,
                 m_priv_host, m_acl_map->global_acl()));
     set_master_access(m_acl_map->global_acl(), m_acl_map->restrictions());
@@ -494,7 +496,8 @@ void Security_context::get_active_roles(THD *thd, List<LEX_USER> &list) {
   @returns Access granted to user for given database
 */
 
-ulong Security_context::db_acl(LEX_CSTRING db, bool use_pattern_scan) const {
+Access_bitmask Security_context::db_acl(LEX_CSTRING db,
+                                        bool use_pattern_scan) const {
   DBUG_TRACE;
   if (m_acl_map == nullptr || db.length == 0) return 0;
 
@@ -502,7 +505,7 @@ ulong Security_context::db_acl(LEX_CSTRING db, bool use_pattern_scan) const {
   Db_access_map::iterator found_acl_it = m_acl_map->db_acls()->find(key);
   if (found_acl_it == m_acl_map->db_acls()->end()) {
     Db_access_map::iterator it = m_acl_map->db_wild_acls()->begin();
-    ulong access = 0;
+    Access_bitmask access = 0;
     for (; it != m_acl_map->db_wild_acls()->end(); ++it) {
       /*
         Do the usual string comparison if partial_revokes is ON,
@@ -527,8 +530,8 @@ ulong Security_context::db_acl(LEX_CSTRING db, bool use_pattern_scan) const {
   }
 }
 
-ulong Security_context::procedure_acl(LEX_CSTRING db,
-                                      LEX_CSTRING procedure_name) {
+Access_bitmask Security_context::procedure_acl(LEX_CSTRING db,
+                                               LEX_CSTRING procedure_name) {
   if (m_acl_map == nullptr)
     return 0;
   else {
@@ -545,7 +548,8 @@ ulong Security_context::procedure_acl(LEX_CSTRING db,
   }
 }
 
-ulong Security_context::function_acl(LEX_CSTRING db, LEX_CSTRING func_name) {
+Access_bitmask Security_context::function_acl(LEX_CSTRING db,
+                                              LEX_CSTRING func_name) {
   if (m_acl_map == nullptr)
     return 0;
   else {
@@ -576,7 +580,7 @@ Grant_table_aggregate Security_context::table_and_column_acls(
   return it->second;
 }
 
-ulong Security_context::table_acl(LEX_CSTRING db, LEX_CSTRING table) {
+Access_bitmask Security_context::table_acl(LEX_CSTRING db, LEX_CSTRING table) {
   if (m_acl_map == nullptr) return 0;
   Grant_table_aggregate aggr = table_and_column_acls(db, table);
   return filter_access(aggr.table_access, db.str ? db.str : "");
@@ -1107,8 +1111,8 @@ void Security_context::init_restrictions(const Restrictions &restrictions) {
 }
 
 bool Security_context::is_access_restricted_on_db(
-    ulong want_access, const std::string &db_name) const {
-  ulong filtered_access = filter_access(want_access, db_name);
+    Access_bitmask want_access, const std::string &db_name) const {
+  const Access_bitmask filtered_access = filter_access(want_access, db_name);
   return (filtered_access != want_access);
 }
 
@@ -1121,12 +1125,12 @@ bool Security_context::is_access_restricted_on_db(
 
   @retval filtered access mask
 */
-ulong Security_context::filter_access(const ulong access,
-                                      const std::string &db_name) const {
-  ulong access_mask = access;
+Access_bitmask Security_context::filter_access(
+    const Access_bitmask access, const std::string &db_name) const {
+  Access_bitmask access_mask = access;
   auto &db_restrictions = m_restrictions.db();
   if (!db_restrictions.is_empty()) {
-    ulong restrictions_mask;
+    Access_bitmask restrictions_mask;
     if (db_restrictions.find(db_name, restrictions_mask))
       access_mask = (access_mask & restrictions_mask) ^ access;
   }
@@ -1184,7 +1188,8 @@ std::pair<bool, bool> Security_context::fetch_global_grant(
   @retval true Success
   @retval false Failure
  */
-bool Security_context::has_table_access(ulong priv, Table_ref *tables) {
+bool Security_context::has_table_access(Access_bitmask priv,
+                                        Table_ref *tables) {
   DBUG_TRACE;
   assert(tables != nullptr);
   TABLE const *table = tables->table;
@@ -1195,7 +1200,7 @@ bool Security_context::has_table_access(ulong priv, Table_ref *tables) {
   table_name.str = table->alias;
   table_name.length = strlen(table->alias);
 
-  ulong acls = master_access({db.str, db.length});
+  Access_bitmask acls = master_access({db.str, db.length});
   if (m_acl_map) {
     if (priv & acls) return true;
 
@@ -1228,7 +1233,8 @@ bool Security_context::has_table_access(ulong priv, Table_ref *tables) {
   @retval true Access to the table is blocked
   @retval false Access to the table is not blocked
  */
-bool Security_context::is_table_blocked(ulong priv, TABLE const *table) {
+bool Security_context::is_table_blocked(Access_bitmask priv,
+                                        TABLE const *table) {
   DBUG_TRACE;
   assert(table != nullptr);
   LEX_CSTRING db, table_name;
@@ -1261,7 +1267,8 @@ bool Security_context::is_table_blocked(ulong priv, TABLE const *table) {
   @retval true Success
   @retval false Failure
  */
-bool Security_context::has_column_access(ulong priv, TABLE const *table,
+bool Security_context::has_column_access(Access_bitmask priv,
+                                         TABLE const *table,
                                          std::vector<std::string> columns) {
   DBUG_TRACE;
   assert(table != nullptr);
