@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -44,7 +45,7 @@
 #include "sql/dd/impl/raw/raw_record.h"           // Raw_record
 #include "sql/dd/impl/sdi_impl.h"                 // sdi read/write functions
 #include "sql/dd/impl/tables/column_type_elements.h"  // Column_type_elements
-#include "sql/dd/impl/tables/columns.h"               // Colummns
+#include "sql/dd/impl/tables/columns.h"               // Columns
 #include "sql/dd/impl/transaction_impl.h"  // Open_dictionary_tables_ctx
 #include "sql/dd/impl/types/abstract_table_impl.h"       // Abstract_table_impl
 #include "sql/dd/impl/types/column_type_element_impl.h"  // Column_type_element_impl
@@ -64,8 +65,9 @@ class Sdi_rcontext;
 class Sdi_wcontext;
 
 static const std::set<String_type> default_valid_option_keys = {
-    "column_format", "geom_type",         "interval_count", "not_secondary",
-    "storage",       "treat_bit_as_char", "is_array"};
+    "column_format", "geom_type", "interval_count",
+    "not_secondary", "storage",   "treat_bit_as_char",
+    "is_array",      "gipk" /* generated implicit primary key column */};
 
 ///////////////////////////////////////////////////////////////////////////
 // Column_impl implementation.
@@ -123,7 +125,7 @@ Column_impl::Column_impl(Abstract_table_impl *table)
       m_is_explicit_collation(false),
       m_column_key(CK_NONE) {}
 
-Column_impl::~Column_impl() {}
+Column_impl::~Column_impl() = default;
 
 ///////////////////////////////////////////////////////////////////////////
 
