@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -83,14 +84,14 @@ static Check_table_intact table_intact;
 const TABLE_FIELD_TYPE event_table_fields[ET_FIELD_COUNT] = {
     {{STRING_WITH_LEN("db")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("name")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
     {{STRING_WITH_LEN("definer")},
      {STRING_WITH_LEN("char(93)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("execute_at")},
      {STRING_WITH_LEN("datetime")},
      {nullptr, 0}},
@@ -141,20 +142,20 @@ const TABLE_FIELD_TYPE event_table_fields[ET_FIELD_COUNT] = {
      {nullptr, 0}},
     {{STRING_WITH_LEN("comment")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("originator")}, {STRING_WITH_LEN("int")}, {nullptr, 0}},
     {{STRING_WITH_LEN("time_zone")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("latin1")}},
     {{STRING_WITH_LEN("character_set_client")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("collation_connection")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("db_collation")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("body_utf8")},
      {STRING_WITH_LEN("longblob")},
      {nullptr, 0}}};
@@ -169,14 +170,14 @@ static const TABLE_FIELD_DEF event_table_def = {ET_FIELD_COUNT,
 static const TABLE_FIELD_TYPE event_table_fields_old[ET_FIELD_COUNT] = {
     {{STRING_WITH_LEN("db")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("name")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("body")}, {STRING_WITH_LEN("longblob")}, {nullptr, 0}},
     {{STRING_WITH_LEN("definer")},
      {STRING_WITH_LEN("char(77)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("execute_at")},
      {STRING_WITH_LEN("datetime")},
      {nullptr, 0}},
@@ -227,20 +228,20 @@ static const TABLE_FIELD_TYPE event_table_fields_old[ET_FIELD_COUNT] = {
      {nullptr, 0}},
     {{STRING_WITH_LEN("comment")},
      {STRING_WITH_LEN("char(64)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("originator")}, {STRING_WITH_LEN("int")}, {nullptr, 0}},
     {{STRING_WITH_LEN("time_zone")},
      {STRING_WITH_LEN("char(64)")},
      {STRING_WITH_LEN("latin1")}},
     {{STRING_WITH_LEN("character_set_client")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("collation_connection")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("db_collation")},
      {STRING_WITH_LEN("char(32)")},
-     {STRING_WITH_LEN("utf8")}},
+     {STRING_WITH_LEN("utf8mb3")}},
     {{STRING_WITH_LEN("body_utf8")},
      {STRING_WITH_LEN("longblob")},
      {nullptr, 0}}};
@@ -501,13 +502,13 @@ static bool migrate_event_to_dd(THD *thd, TABLE *event_table) {
 
   dd::upgrade::Routine_event_context_guard event_ctx_guard(thd);
 
-  thd->variables.sql_mode = (sql_mode_t)(
-      event_table->field[ET_FIELD_SQL_MODE]->val_int() & MODE_ALLOWED_MASK);
+  thd->variables.sql_mode =
+      (sql_mode_t)(event_table->field[ET_FIELD_SQL_MODE]->val_int() &
+                   MODE_ALLOWED_MASK);
 
   // Holders for user name and host name used in parse user.
   char definer_user_name_holder[USERNAME_LENGTH + 1];
   char definer_host_name_holder[HOSTNAME_LENGTH + 1];
-  memset(&user_info, 0, sizeof(LEX_USER));
   user_info.user = {definer_user_name_holder, USERNAME_LENGTH};
   user_info.host = {definer_host_name_holder, HOSTNAME_LENGTH};
 
@@ -569,7 +570,7 @@ bool migrate_events_to_dd(THD *thd) {
   MEM_ROOT records_mem_root;
   Thd_mem_root_guard root_guard(thd, &records_mem_root);
 
-  TABLE_LIST tables("mysql", "event", TL_READ);
+  Table_ref tables("mysql", "event", TL_READ);
   auto table_list = &tables;
 
   if (open_and_lock_tables(thd, table_list, flags, &prelocking_strategy)) {
