@@ -1,18 +1,19 @@
 #ifndef ITEM_JSON_FUNC_INCLUDED
 #define ITEM_JSON_FUNC_INCLUDED
 
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,16 +39,17 @@
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysql_time.h"
-#include "prealloced_array.h"  // Prealloced_array
+#include "prealloced_array.h"      // Prealloced_array
+#include "sql-common/json_path.h"  // Json_path
 #include "sql/enum_query_type.h"
 #include "sql/field.h"
 #include "sql/item.h"
 #include "sql/item_cmpfunc.h"
 #include "sql/item_func.h"
 #include "sql/item_strfunc.h"    // Item_str_func
-#include "sql/json_path.h"       // Json_path
 #include "sql/mem_root_array.h"  // Mem_root_array
 #include "sql/parse_location.h"  // POS
+#include "sql/psi_memory_key.h"  // key_memory_JSON
 #include "sql_string.h"
 
 class Json_schema_validator;
@@ -174,7 +176,7 @@ class Item_json_func : public Item_func {
     @param parent_args  arguments to forward to Item_func's constructor
   */
   template <typename... Args>
-  Item_json_func(THD *thd, Args &&... parent_args)
+  Item_json_func(THD *thd, Args &&...parent_args)
       : Item_func(std::forward<Args>(parent_args)...),
         m_path_cache(thd, arg_count) {
     set_data_type_json();
@@ -499,15 +501,8 @@ class Item_typecast_json final : public Item_json_func {
 class Item_func_json_length final : public Item_int_func {
   String m_doc_value;
 
-  // Cache for constant path expressions
-  Json_path_cache m_path_cache;
-
  public:
-  Item_func_json_length(THD *thd, const POS &pos, Item *a)
-      : Item_int_func(pos, a), m_path_cache(thd, 1) {}
-
-  Item_func_json_length(THD *thd, const POS &pos, Item *a, Item *b)
-      : Item_int_func(pos, a, b), m_path_cache(thd, 2) {}
+  Item_func_json_length(const POS &pos, Item *doc) : Item_int_func(pos, doc) {}
 
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
@@ -519,8 +514,6 @@ class Item_func_json_length final : public Item_int_func {
   const char *func_name() const override { return "json_length"; }
 
   longlong val_int() override;
-
-  void cleanup() override;
 };
 
 /**
@@ -601,7 +594,7 @@ class Item_func_json_extract final : public Item_json_func {
 class Item_func_modify_json_in_path : public Item_json_func {
  protected:
   template <typename... Args>
-  explicit Item_func_modify_json_in_path(Args &&... parent_args)
+  explicit Item_func_modify_json_in_path(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...) {
     // The function does not necessarily return NULL when an argument is NULL.
     // It returns NULL only if the first argument is NULL, or if one of the JSON
@@ -672,9 +665,10 @@ class Item_func_json_set_replace : public Item_func_modify_json_in_path {
 
  protected:
   template <typename... Args>
-  explicit Item_func_json_set_replace(bool json_set, Args &&... parent_args)
+  explicit Item_func_json_set_replace(bool json_set, Args &&...parent_args)
       : Item_func_modify_json_in_path(std::forward<Args>(parent_args)...),
-        m_json_set(json_set) {}
+        m_json_set(json_set),
+        m_path(key_memory_JSON) {}
 
  public:
   bool val_json(Json_wrapper *wr) override;
@@ -686,7 +680,7 @@ class Item_func_json_set_replace : public Item_func_modify_json_in_path {
 class Item_func_json_set final : public Item_func_json_set_replace {
  public:
   template <typename... Args>
-  explicit Item_func_json_set(Args &&... parent_args)
+  explicit Item_func_json_set(Args &&...parent_args)
       : Item_func_json_set_replace(true, std::forward<Args>(parent_args)...) {}
 
   const char *func_name() const override { return "json_set"; }
@@ -698,7 +692,7 @@ class Item_func_json_set final : public Item_func_json_set_replace {
 class Item_func_json_replace final : public Item_func_json_set_replace {
  public:
   template <typename... Args>
-  explicit Item_func_json_replace(Args &&... parent_args)
+  explicit Item_func_json_replace(Args &&...parent_args)
       : Item_func_json_set_replace(false, std::forward<Args>(parent_args)...) {}
 
   const char *func_name() const override { return "json_replace"; }
@@ -710,7 +704,7 @@ class Item_func_json_replace final : public Item_func_json_set_replace {
 class Item_func_json_array final : public Item_json_func {
  public:
   template <typename... Args>
-  explicit Item_func_json_array(Args &&... parent_args)
+  explicit Item_func_json_array(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...) {
     // Does not return NULL on NULL input. A NULL argument is interpreted as the
     // JSON null literal.
@@ -772,7 +766,7 @@ class Item_func_json_search : public Item_json_func {
     @param parent_args arguments to pass to Item_json_func's constructor
   */
   template <typename... Args>
-  Item_func_json_search(Args &&... parent_args)
+  Item_func_json_search(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...),
         m_cached_ooa(ooa_uninitialized) {}
 
@@ -804,7 +798,7 @@ class Item_func_json_remove : public Item_json_func {
 
  public:
   template <typename... Args>
-  Item_func_json_remove(Args &&... parent_args)
+  Item_func_json_remove(Args &&...parent_args)
       : Item_json_func(std::forward<Args>(parent_args)...) {}
 
   const char *func_name() const override { return "json_remove"; }
@@ -901,6 +895,7 @@ class Item_func_json_quote : public Item_str_func {
 */
 class Item_func_json_unquote : public Item_str_func {
   String m_value;
+  String m_conversion_buffer;
 
  public:
   Item_func_json_unquote(const POS &pos, PT_item_list *a)
@@ -951,6 +946,8 @@ class Item_func_json_storage_size final : public Item_int_func {
 
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_JSON)) return true;
+    if (Item_int_func::resolve_type(thd)) return true;
+    set_nullable(true);
     return false;
   }
 
@@ -1175,7 +1172,7 @@ class Item_func_json_value final : public Item_func {
 
 /**
   Turn a GEOMETRY value into a JSON value per the GeoJSON specification
-  revison 1.0. This method is implemented in item_geofunc.cc.
+  revision 1.0. This method is implemented in item_geofunc.cc.
 
   @param[in,out] wr The wrapper to be stuffed with the JSON value.
   @param[in]     swkb The source GEOMETRY value.
@@ -1245,8 +1242,9 @@ bool get_json_object_member_name(const THD *thd, Item *arg_item, String *value,
                                  size_t *safe_length);
 using Json_dom_ptr = std::unique_ptr<Json_dom>;
 
-bool parse_json(const String &res, uint arg_idx, const char *func_name,
-                Json_dom_ptr *dom, bool require_str_or_json, bool *parse_error);
+bool parse_json(const String &res, Json_dom_ptr *dom, bool require_str_or_json,
+                const JsonParseErrorHandler &error_handler,
+                const JsonDocumentDepthHandler &depth_handler);
 
 typedef Prealloced_array<size_t, 16> Sorted_index_array;
 bool sort_and_remove_dups(const Json_wrapper &orig, Sorted_index_array *v);
