@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -82,11 +83,12 @@
 #include "sql/sql_select.h"
 #include "sql/table.h"
 
-static bool find_key_for_maxmin(bool max_fl, TABLE_REF *ref,
+static bool find_key_for_maxmin(bool max_fl, Index_lookup *ref,
                                 Item_field *item_field, Item *cond,
                                 uint *range_fl, uint *key_prefix_length);
-static bool reckey_in_range(bool max_fl, TABLE_REF *ref, Item_field *item_field,
-                            Item *cond, uint range_fl, uint prefix_len);
+static bool reckey_in_range(bool max_fl, Index_lookup *ref,
+                            Item_field *item_field, Item *cond, uint range_fl,
+                            uint prefix_len);
 static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond);
 
 /**
@@ -98,9 +100,9 @@ static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond);
 
     @retval Product of number of rows in all tables. ULLONG_MAX for error.
 */
-ulonglong get_exact_record_count(TABLE_LIST *tables) {
+static ulonglong get_exact_record_count(Table_ref *tables) {
   ulonglong count = 1;
-  for (TABLE_LIST *tl = tables; tl; tl = tl->next_leaf) {
+  for (Table_ref *tl = tables; tl; tl = tl->next_leaf) {
     ha_rows tmp = 0;
     int error = tl->table->file->ha_records(&tmp);
     if (error != 0) return ULLONG_MAX;
@@ -123,7 +125,7 @@ ulonglong get_exact_record_count(TABLE_LIST *tables) {
     HA_ERR_...      Otherwise
 */
 
-static int get_index_min_value(TABLE *table, TABLE_REF *ref,
+static int get_index_min_value(TABLE *table, Index_lookup *ref,
                                Item_field *item_field, uint range_fl,
                                uint prefix_len) {
   int error;
@@ -207,7 +209,7 @@ static int get_index_min_value(TABLE *table, TABLE_REF *ref,
     HA_ERR_...      Otherwise
 */
 
-static int get_index_max_value(TABLE *table, TABLE_REF *ref, uint range_fl) {
+static int get_index_max_value(TABLE *table, Index_lookup *ref, uint range_fl) {
   return (ref->key_length
               ? table->file->ha_index_read_map(
                     table->record[0], ref->key_buff,
@@ -295,7 +297,7 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
   // The set of tables in the join, excluding the inner tables of outer join
   table_map used_tables = 0;
 
-  TABLE_LIST *tables = select->leaf_tables;
+  Table_ref *tables = select->leaf_tables;
 
   int error;
 
@@ -321,7 +323,7 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
     Analyze outer join dependencies, and, if possible, compute the number
     of returned rows.
   */
-  for (TABLE_LIST *tl = tables; tl; tl = tl->next_leaf) {
+  for (Table_ref *tl = tables; tl; tl = tl->next_leaf) {
     // Don't replace expression on a table that is part of an outer join
     if (tl->is_inner_table_of_outer_join()) {
       inner_tables |= tl->map();
@@ -416,7 +418,7 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
           /*
             For result count of full-text search: If
             1. it is a single table query,
-            2. the WHERE condition is a single MATCH expresssion,
+            2. the WHERE condition is a single MATCH expression,
             3. the table engine can provide the row count from FTS result, and
             4. the expr in COUNT(expr) can not be NULL,
             we do the full-text search now, and replace with the actual count.
@@ -464,14 +466,22 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
           Item *expr = item_sum->get_arg(0)->real_item();
           if (expr->type() == Item::FIELD_ITEM) {
             uchar key_buff[MAX_KEY_LENGTH];
-            TABLE_REF ref;
+            Index_lookup ref;
             uint range_fl, prefix_len;
 
             ref.key_buff = key_buff;
             Item_field *item_field = down_cast<Item_field *>(expr);
-            TABLE_LIST *tr = item_field->table_ref;
+            Table_ref *tr = item_field->table_ref;
             TABLE *table = tr->table;
-
+            /*
+              If table already has been read, we may use the values already
+              in record buffer. However, the logic around this is a bit unclear,
+              thus we skip this possible optimization for now.
+            */
+            if (table->const_table) {
+              aggr_impossible = true;
+              break;
+            }
             /*
               We must not have accessed this table instance yet, because
               it must be private to this query block, as we already ensured
@@ -547,7 +557,8 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
               return true;
             }
             removed_tables |= tr->map();
-          } else if (!expr->const_item() || conds || !have_exact_count) {
+          } else if (!expr->const_for_execution() || conds != nullptr ||
+                     !have_exact_count) {
             /*
               We get here if the aggregate function is not based on a field.
               Example: "SELECT MAX(1) FROM table ..."
@@ -606,7 +617,8 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
     aggr_impossible = true;  // We didn't remove all tables
 
   *decision = aggr_impossible ? AGGR_REGULAR
-                              : aggr_delayed ? AGGR_DELAYED : AGGR_COMPLETE;
+              : aggr_delayed  ? AGGR_DELAYED
+                              : AGGR_COMPLETE;
   return false;
 }
 
@@ -615,28 +627,23 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
 
   @param func_item        Predicate item
   @param[out] args        Here we store the field followed by constants
-  @param[out] inv_order   Is set to 1 if the predicate is of the form
+  @param[out] inv_order   Is set to true if the predicate is of the form
                           'const op field'
 
-  @retval
-    0        func_item is a simple predicate: a field is compared with
-    constants
-  @retval
-    1        Otherwise
+  @returns true if function is a simple predicate, false otherwise.
 */
 
-bool simple_pred(Item_func *func_item, Item **args, bool *inv_order) {
+bool is_simple_predicate(Item_func *func_item, Item **args, bool *inv_order) {
   Item *item;
   *inv_order = false;
   switch (func_item->argument_count()) {
     case 0:
       /* MULT_EQUAL_FUNC */
       {
-        Item_equal *item_equal = (Item_equal *)func_item;
-        Item_equal_iterator it(*item_equal);
-        args[0] = it++;
-        if (it++) return false;
-        if (!(args[1] = item_equal->get_const())) return false;
+        Item_equal *item_equal = down_cast<Item_equal *>(func_item);
+        args[0] = item_equal->get_first();
+        if (item_equal->members() > 1) return false;
+        if (!(args[1] = item_equal->const_arg())) return false;
       }
       break;
     case 1:
@@ -651,9 +658,9 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order) {
       if (item->type() == Item::FIELD_ITEM) {
         args[0] = item;
         item = func_item->arguments()[1];
-        if (!item->const_item()) return false;
+        if (!item->const_for_execution()) return false;
         args[1] = item;
-      } else if (item->const_item()) {
+      } else if (item->const_for_execution()) {
         args[1] = item;
         item = func_item->arguments()[1];
         if (item->type() != Item::FIELD_ITEM) return false;
@@ -665,15 +672,13 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order) {
     case 3:
       /* field BETWEEN const AND const */
       item = func_item->arguments()[0];
-      if (item->type() == Item::FIELD_ITEM) {
-        args[0] = item;
-        for (int i = 1; i <= 2; i++) {
-          item = func_item->arguments()[i];
-          if (!item->const_item()) return false;
-          args[i] = item;
-        }
-      } else
-        return false;
+      if (item->type() != Item::FIELD_ITEM) return false;
+      args[0] = item;
+      for (int i = 1; i <= 2; i++) {
+        item = func_item->arguments()[i];
+        if (!item->const_for_execution()) return false;
+        args[i] = item;
+      }
   }
   return true;
 }
@@ -704,7 +709,7 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order) {
    SELECT MIN(a) FROM t1 WHERE a > 3 AND a > 5;
    @endcode
 
-   the algorithm will recurse over the conjuction, storing first a 3 in the
+   the algorithm will recurse over the conjunction, storing first a 3 in the
    field. In the next recursive invocation the expression a > 5 is evaluated
    as 3 > 5 (Due to the dual nature of Field objects as value carriers and
    field identifiers), which will obviously fail, leading to 5 being stored in
@@ -733,7 +738,7 @@ bool simple_pred(Item_func *func_item, Item **args, bool *inv_order) {
     true     We can use the index to get MIN/MAX value
 */
 
-static bool matching_cond(bool max_fl, TABLE_REF *ref, KEY *keyinfo,
+static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
                           KEY_PART_INFO *field_part, Item *cond, table_map map,
                           key_part_map *key_part_used, uint *range_fl,
                           uint *prefix_len) {
@@ -771,7 +776,8 @@ static bool matching_cond(bool max_fl, TABLE_REF *ref, KEY *keyinfo,
 
   switch (((Item_func *)cond)->functype()) {
     case Item_func::ISNULL_FUNC:
-      is_null = true; /* fall through */
+      is_null = true;
+      [[fallthrough]];
     case Item_func::EQ_FUNC:
       eq_type = true;
       break;
@@ -779,12 +785,14 @@ static bool matching_cond(bool max_fl, TABLE_REF *ref, KEY *keyinfo,
       eq_type = is_null_safe_eq = true;
       break;
     case Item_func::LT_FUNC:
-      noeq_type = true; /* fall through */
+      noeq_type = true;
+      [[fallthrough]];
     case Item_func::LE_FUNC:
       less_fl = true;
       break;
     case Item_func::GT_FUNC:
-      noeq_type = true; /* fall through */
+      noeq_type = true;
+      [[fallthrough]];
     case Item_func::GE_FUNC:
       break;
     case Item_func::BETWEEN:
@@ -805,7 +813,8 @@ static bool matching_cond(bool max_fl, TABLE_REF *ref, KEY *keyinfo,
   bool inv;
 
   /* Test if this is a comparison of a field and constant */
-  if (!simple_pred((Item_func *)cond, args, &inv)) return false;
+  if (!is_simple_predicate(down_cast<Item_func *>(cond), args, &inv))
+    return false;
 
   if (!is_null_safe_eq && !is_null &&
       (args[1]->is_null() || (between && args[2]->is_null())))
@@ -877,13 +886,24 @@ static bool matching_cond(bool max_fl, TABLE_REF *ref, KEY *keyinfo,
       Item *value = args[between && max_fl ? 2 : 1];
 
       /*
-        A perfect save is neccessary. Truncated / incorrect value can result
+        A perfect save is necessary. Truncated / incorrect value can result
         in an incorrect index lookup. Truncation of trailing space is ignored
-        since that is expected for strings even in other cases.
+        for strings with a PAD SPACE collation. (When trailing space has been
+        removed, TYPE_NOTE_TRUNCATED is returned. Not to be confused with
+        TYPE_WARN_TRUNCATED, which is returned when non-space has been removed.)
+
+        TODO(khatlen): It might be better if string literals that need
+        truncation are handled during constant folding, so that we only need to
+        check for TYPE_OK here. analyze_field_constant() does not yet handle
+        string constants.
       */
       type_conversion_status retval =
           value->save_in_field_no_warnings(part->field, true);
-      if (!(retval == TYPE_OK || retval == TYPE_NOTE_TRUNCATED)) return false;
+      if (!(retval == TYPE_OK ||
+            (retval == TYPE_NOTE_TRUNCATED && part->field->is_text_key_type() &&
+             part->field->charset()->pad_attribute == PAD_SPACE))) {
+        return false;
+      }
 
       if (part->null_bit) *key_ptr++ = (uchar)(part->field->is_null());
       part->field->get_key_image(key_ptr, part->length, Field::itRAW);
@@ -943,14 +963,11 @@ static bool matching_cond(bool max_fl, TABLE_REF *ref, KEY *keyinfo,
     which must be reset after index is used!
     (This can only happen when function returns 1)
 
-  @retval
-    0   Index can not be used to optimize MIN(field)/MAX(field)
-  @retval
-    1   Can use key to optimize MIN()/MAX().
-    In this case ref, range_fl and prefix_len are updated
+  @returns true if index can be used to optimize MIN()/MAX(), false otherwise.
+           If true, ref, range_fl and prefix_len are updated
 */
 
-static bool find_key_for_maxmin(bool max_fl, TABLE_REF *ref,
+static bool find_key_for_maxmin(bool max_fl, Index_lookup *ref,
                                 Item_field *item_field, Item *cond,
                                 uint *range_fl, uint *prefix_len) {
   Field *const field = item_field->field;
@@ -1037,46 +1054,45 @@ static bool find_key_for_maxmin(bool max_fl, TABLE_REF *ref,
 /**
   Check whether found key is in range specified by conditions.
 
-  @param[in] max_fl         0 for MIN(field) / 1 for MAX(field)
+  @param[in] max_fl         false for MIN(field) / true for MAX(field)
   @param[in] ref            Reference to the key value and info
   @param[in] item_field     Item representing field used in MIN/MAX expression
-  @param[in] cond           WHERE condition
+  @param[in] cond           Condition to check
+                            nullptr means that all keys are within the range.
   @param[in] range_fl       Says whether there is a condition to to be checked
   @param[in] prefix_len     Length of the constant part of the key
 
-  @retval
-    false    ok
-  @retval
-    true     WHERE was not true for the found row
+  @returns true if the condition is not true for the found row, false otherwise.
 */
 
-static bool reckey_in_range(bool max_fl, TABLE_REF *ref, Item_field *item_field,
-                            Item *cond, uint range_fl, uint prefix_len) {
+static bool reckey_in_range(bool max_fl, Index_lookup *ref,
+                            Item_field *item_field, Item *cond, uint range_fl,
+                            uint prefix_len) {
   if (key_cmp_if_same(item_field->field->table, ref->key_buff, ref->key,
                       prefix_len))
     return true;
-  if (!cond || (range_fl & (max_fl ? NO_MIN_RANGE : NO_MAX_RANGE)))
-    return false;
+  if (cond == nullptr) return false;
+  // Constant conditions which were not evaluated earlier (in optimize_cond())
+  // should be evaluated now.
+  if (cond->const_for_execution()) return (cond->val_int() == 0);
+  if (range_fl & (max_fl ? NO_MIN_RANGE : NO_MAX_RANGE)) return false;
   return maxmin_in_range(max_fl, item_field, cond);
 }
 
 /**
   Check whether {MAX|MIN}(field) is in range specified by conditions.
 
-  @param[in] max_fl          0 for MIN(field) / 1 for MAX(field)
-  @param[in] item_field      Item representing field used in MIN/MAX expression
-  @param[in] cond            WHERE condition
+  @param[in] max_fl      false for MIN(field) / true for MAX(field)
+  @param[in] item_field  Item representing field used in MIN/MAX expression
+  @param[in] cond        Condition to check
 
-  @retval
-    false    ok
-  @retval
-    true     WHERE was not true for the found row
+  @returns true if condition is not true for the found row, otherwise false.
 */
 
 static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond) {
   /* If AND/OR condition */
   if (cond->type() == Item::COND_ITEM) {
-    List_iterator_fast<Item> li(*((Item_cond *)cond)->argument_list());
+    List_iterator_fast<Item> li(*down_cast<Item_cond *>(cond)->argument_list());
     Item *item;
     while ((item = li++)) {
       if (maxmin_in_range(max_fl, item_field, item)) return true;
@@ -1084,20 +1100,21 @@ static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond) {
     return false;
   }
 
-  if (cond->used_tables() != item_field->table_ref->map()) return false;
+  // Check that condition actually references the specific table
+  if ((cond->used_tables() & item_field->table_ref->map()) == 0) return false;
   bool less_fl = false;
-  switch (((Item_func *)cond)->functype()) {
+  switch (down_cast<Item_func *>(cond)->functype()) {
     case Item_func::BETWEEN:
-      return cond->val_int() == 0;  // Return 1 if WHERE is false
+      return cond->val_int() == 0;
     case Item_func::LT_FUNC:
     case Item_func::LE_FUNC:
       less_fl = true;
-      // Fall through
+      [[fallthrough]];
     case Item_func::GT_FUNC:
     case Item_func::GE_FUNC: {
-      Item *item = ((Item_func *)cond)->arguments()[1];
+      Item *item = down_cast<Item_func *>(cond)->arguments()[1];
       /* In case of 'const op item' we have to swap the operator */
-      if (!item->const_item()) less_fl = !less_fl;
+      if (!item->const_for_execution()) less_fl = !less_fl;
       /*
         We only have to check the expression if we are using an expression like
         SELECT MAX(b) FROM t1 WHERE a=const AND b>const
@@ -1105,7 +1122,7 @@ static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond) {
         SELECT MAX(b) FROM t1 WHERE a=const AND b<const
       */
       if (max_fl != less_fl)
-        return cond->val_int() == 0;  // Return 1 if WHERE is false
+        return cond->val_int() == 0;  // Return true if condition is false
       return false;
     }
     case Item_func::EQ_FUNC:
