@@ -1,15 +1,16 @@
-/* Copyright (c) 2019, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2019, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,11 +23,16 @@
 
 #include <algorithm>
 
+#include "mutex_lock.h"            // MUTEX_LOCK
+#include "mysql/psi/mysql_cond.h"  // mysql_cond_timedwait
 #include "sql/binlog.h"
-#include "sql/debug_sync.h"  // DEBUG_SYNC
+#include "sql/binlog/group_commit/bgc_ticket_manager.h"  // Bgc_ticket_manager
+#include "sql/debug_sync.h"                              // DEBUG_SYNC
+#include "sql/mysqld.h"       //PSI_stage_info stage_wait_on_commit_ticket
+#include "sql/raii/sentry.h"  // raii::Sentry<>
 #include "sql/rpl_commit_stage_manager.h"
-#include "sql/rpl_rli_pdb.h"  // Slave_worker                    // Slave_worker
-#include "sql/rpl_slave_commit_order_manager.h"  // Commit_order_manager
+#include "sql/rpl_replica_commit_order_manager.h"  // Commit_order_manager
+#include "sql/rpl_rli_pdb.h"                       // Slave_worker
 
 class Slave_worker;
 class Commit_order_manager;
@@ -95,9 +101,12 @@ std::pair<bool, THD *> Commit_stage_manager::Mutex_queue::pop_front() {
 void Commit_stage_manager::init(PSI_mutex_key key_LOCK_flush_queue,
                                 PSI_mutex_key key_LOCK_sync_queue,
                                 PSI_mutex_key key_LOCK_commit_queue,
+                                PSI_mutex_key key_LOCK_after_commit_queue,
                                 PSI_mutex_key key_LOCK_done,
+                                PSI_mutex_key key_LOCK_wait_for_group_turn,
                                 PSI_cond_key key_COND_done,
-                                PSI_cond_key key_COND_flush_queue) {
+                                PSI_cond_key key_COND_flush_queue,
+                                PSI_cond_key key_COND_wait_for_group_turn) {
   if (m_is_initialized) return;
   m_is_initialized = true;
 
@@ -105,6 +114,8 @@ void Commit_stage_manager::init(PSI_mutex_key key_LOCK_flush_queue,
   mysql_cond_init(key_COND_done, &m_stage_cond_binlog);
   mysql_cond_init(key_COND_done, &m_stage_cond_commit_order);
   mysql_cond_init(key_COND_flush_queue, &m_stage_cond_leader);
+  mysql_cond_init(key_COND_wait_for_group_turn,
+                  &this->m_cond_wait_for_ticket_turn);
 #ifndef NDEBUG
   leader_thd = nullptr;
 
@@ -115,8 +126,8 @@ void Commit_stage_manager::init(PSI_mutex_key key_LOCK_flush_queue,
 #endif
 
   /**
-    Initialize mutex for flush, sync and commit stage queue. The binlog flush
-    stage and commit order flush stage share same mutex.
+    Initialize mutex for flush, sync, commit and after commit stage queue. The
+    binlog flush stage and commit order flush stage share same mutex.
   */
   mysql_mutex_init(key_LOCK_flush_queue, &m_queue_lock[BINLOG_FLUSH_STAGE],
                    MY_MUTEX_INIT_FAST);
@@ -124,10 +135,15 @@ void Commit_stage_manager::init(PSI_mutex_key key_LOCK_flush_queue,
                    MY_MUTEX_INIT_FAST);
   mysql_mutex_init(key_LOCK_commit_queue, &m_queue_lock[COMMIT_STAGE],
                    MY_MUTEX_INIT_FAST);
+  mysql_mutex_init(key_LOCK_after_commit_queue,
+                   &m_queue_lock[AFTER_COMMIT_STAGE], MY_MUTEX_INIT_FAST);
+  mysql_mutex_init(key_LOCK_wait_for_group_turn,
+                   &this->m_lock_wait_for_ticket_turn, MY_MUTEX_INIT_FAST);
 
   m_queue[BINLOG_FLUSH_STAGE].init(&m_queue_lock[BINLOG_FLUSH_STAGE]);
   m_queue[SYNC_STAGE].init(&m_queue_lock[SYNC_STAGE]);
   m_queue[COMMIT_STAGE].init(&m_queue_lock[COMMIT_STAGE]);
+  m_queue[AFTER_COMMIT_STAGE].init(&m_queue_lock[AFTER_COMMIT_STAGE]);
   m_queue[COMMIT_ORDER_FLUSH_STAGE].init(&m_queue_lock[BINLOG_FLUSH_STAGE]);
 }
 
@@ -135,12 +151,88 @@ void Commit_stage_manager::deinit() {
   if (!m_is_initialized) return;
   m_is_initialized = false;
 
-  for (size_t i = 0; i < STAGE_COUNTER - 1; ++i)
-    mysql_mutex_destroy(&m_queue_lock[i]);
+  mysql_mutex_destroy(&m_queue_lock[BINLOG_FLUSH_STAGE]);
+  mysql_mutex_destroy(&m_queue_lock[SYNC_STAGE]);
+  mysql_mutex_destroy(&m_queue_lock[COMMIT_STAGE]);
+  mysql_mutex_destroy(&m_queue_lock[AFTER_COMMIT_STAGE]);
+
   mysql_cond_destroy(&m_stage_cond_binlog);
   mysql_cond_destroy(&m_stage_cond_commit_order);
   mysql_cond_destroy(&m_stage_cond_leader);
   mysql_mutex_destroy(&m_lock_done);
+  mysql_cond_destroy(&this->m_cond_wait_for_ticket_turn);
+  mysql_mutex_destroy(&this->m_lock_wait_for_ticket_turn);
+}
+
+bool Commit_stage_manager::is_ticket_on_its_turn_and_back_ticket_incremented(
+    THD *thd) const {
+  if (!thd->is_applier_thread()) {
+    assert(false);
+    return false;
+  }
+
+  auto &ticket_manager = binlog::Bgc_ticket_manager::instance();
+  auto &ticket_ctx = thd->rpl_thd_ctx.binlog_group_commit_ctx();
+  binlog::BgcTicket ticket(ticket_ctx.get_session_ticket());
+
+  if (ticket == ticket_manager.get_front_ticket() &&
+      ticket < ticket_manager.get_back_ticket()) {
+    return true;
+  }
+
+  return false;
+}
+
+void Commit_stage_manager::wait_for_ticket_turn(THD *thd,
+                                                bool update_ticket_manager) {
+  auto &ticket_ctx = thd->rpl_thd_ctx.binlog_group_commit_ctx();
+  if (ticket_ctx.has_waited()) return;
+
+  auto &ticket_manager = binlog::Bgc_ticket_manager::instance();
+  binlog::BgcTicket ticket(ticket_ctx.get_session_ticket());
+
+  CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP("before_wait_on_ticket");
+  // Check, first, if the session ticket is already being processed, to avoid
+  // acquiring the mutex
+  if (ticket != ticket_manager.get_front_ticket() &&
+      ticket > ticket_manager.get_coalesced_ticket() && !thd->killed) {
+    CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP("inside_wait_on_ticket");
+    PSI_stage_info old_stage;
+    {
+      MUTEX_LOCK(guard, &this->m_lock_wait_for_ticket_turn);
+      thd->ENTER_COND(&this->m_cond_wait_for_ticket_turn,
+                      &this->m_lock_wait_for_ticket_turn,
+                      &stage_wait_on_commit_ticket, &old_stage);
+      struct timespec abstime;
+      while (ticket != ticket_manager.get_front_ticket() &&
+             ticket > ticket_manager.get_coalesced_ticket() && !thd->killed) {
+        // in rare cases View Changes cause ticket changes with no broadcast
+        set_timespec(&abstime, 1);
+        mysql_cond_timedwait(&this->m_cond_wait_for_ticket_turn,
+                             &this->m_lock_wait_for_ticket_turn, &abstime);
+      }
+    }
+    thd->EXIT_COND(&old_stage);
+  }
+
+#ifndef NDEBUG
+  if (Binlog_group_commit_ctx::manual_ticket_setting()->load()) {
+    assert(ticket >= ticket_manager.get_coalesced_ticket());
+  }
+#endif
+
+  if (update_ticket_manager) {
+    this->update_session_ticket_state(thd);
+  }
+}
+
+bool Commit_stage_manager::append_to(StageID stage, THD *thd) {
+  this->wait_for_ticket_turn(thd, false);
+  raii::Sentry<> _ticket_guard{
+      [thd, this]() -> void { this->update_session_ticket_state(thd); }};
+
+  lock_queue(stage);
+  return m_queue[stage].append(thd);
 }
 
 bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
@@ -152,8 +244,8 @@ bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
   DBUG_PRINT("debug",
              ("Enqueue 0x%llx to queue for stage %d", (ulonglong)thd, stage));
 
-  lock_queue(stage);
-  bool leader = m_queue[stage].append(thd);
+  thd->rpl_thd_ctx.binlog_group_commit_ctx().assign_ticket();
+  bool leader = this->append_to(stage, thd);
 
   /*
    if its FLUSH stage queue (BINLOG_FLUSH_STAGE or COMMIT_ORDER_FLUSH_STAGE)
@@ -208,20 +300,11 @@ bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
     Commit_order_manager::finish_one(thd);
   }
 
-  /*
-    We do not need to unlock the stage_mutex if it is LOCK_log when rotating
-    binlog caused by logging incident log event, since it should be held
-    always during rotation.
-  */
-  bool need_unlock_stage_mutex =
-      !(mysql_bin_log.is_rotating_caused_by_incident &&
-        stage_mutex == mysql_bin_log.get_log_lock());
-
   /*
     The stage mutex can be nullptr if we are enrolling for the first
     stage.
   */
-  if (stage_mutex && need_unlock_stage_mutex) mysql_mutex_unlock(stage_mutex);
+  if (stage_mutex) mysql_mutex_unlock(stage_mutex);
 
 #ifndef NDEBUG
   DBUG_PRINT("info", ("This is a leader thread: %d (0=n 1=y)", leader));
@@ -231,12 +314,22 @@ bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
   switch (stage) {
     case BINLOG_FLUSH_STAGE:
       DEBUG_SYNC(thd, "bgc_after_enrolling_for_flush_stage");
+      CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP(
+          "bgc_after_enrolling_for_flush_stage");
       break;
     case SYNC_STAGE:
       DEBUG_SYNC(thd, "bgc_after_enrolling_for_sync_stage");
+      CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP(
+          "bgc_after_enrolling_for_sync_stage");
       break;
     case COMMIT_STAGE:
       DEBUG_SYNC(thd, "bgc_after_enrolling_for_commit_stage");
+      CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP("after_writing_to_tc_log");
+      break;
+    case AFTER_COMMIT_STAGE:
+      DEBUG_SYNC(thd, "bgc_after_enrolling_for_after_commit_stage");
+      CONDITIONAL_SYNC_POINT_FOR_TIMESTAMP(
+          "bgc_after_enrolling_for_after_commit_stage");
       break;
     case COMMIT_ORDER_FLUSH_STAGE:
       break;
@@ -263,7 +356,7 @@ bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
       With setting the status the follower ensures it won't execute anything
       including thread-specific code.
     */
-    thd->get_transaction()->m_flags.ready_preempt = 1;
+    thd->get_transaction()->m_flags.ready_preempt = true;
     if (leader_await_preempt_status) mysql_cond_signal(&m_cond_preempt);
 #endif
     while (thd->tx_commit_pending) {
@@ -283,19 +376,8 @@ bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
     DEBUG_SYNC(thd, "bgc_between_flush_and_sync");
 #endif
 
-  bool need_lock_enter_mutex = false;
   if (leader && enter_mutex != nullptr) {
-    /*
-      We do not lock the enter_mutex if it is LOCK_log when rotating binlog
-      caused by logging incident log event, since it is already locked.
-    */
-    need_lock_enter_mutex = !(mysql_bin_log.is_rotating_caused_by_incident &&
-                              enter_mutex == mysql_bin_log.get_log_lock());
-
-    if (need_lock_enter_mutex)
-      mysql_mutex_lock(enter_mutex);
-    else
-      mysql_mutex_assert_owner(enter_mutex);
+    mysql_mutex_lock(enter_mutex);
   }
 
   if (stage == COMMIT_ORDER_FLUSH_STAGE) {
@@ -304,7 +386,7 @@ bool Commit_stage_manager::enroll_for(StageID stage, THD *thd,
     lock_queue(stage);
 
     if (!m_queue[BINLOG_FLUSH_STAGE].is_empty()) {
-      if (need_lock_enter_mutex) mysql_mutex_unlock(enter_mutex);
+      mysql_mutex_unlock(enter_mutex);
 
       THD *binlog_leader = m_queue[BINLOG_FLUSH_STAGE].get_leader();
       binlog_leader->tx_commit_pending = false;
@@ -404,6 +486,7 @@ void Commit_stage_manager::signal_done(THD *queue, StageID stage) {
 
   for (THD *thd = queue; thd; thd = thd->next_to_commit) {
     thd->tx_commit_pending = false;
+    thd->rpl_thd_ctx.binlog_group_commit_ctx().reset();
   }
 
   /* if thread belong to commit order wake only commit order queue threads */
@@ -415,6 +498,65 @@ void Commit_stage_manager::signal_done(THD *queue, StageID stage) {
   mysql_mutex_unlock(&m_lock_done);
 }
 
+void Commit_stage_manager::signal_end_of_ticket(bool force) {
+  auto &ticket_manager = binlog::Bgc_ticket_manager::instance();
+  // Check, first, if there are any tickets other than the active, to avoid
+  // taking the mutex.
+  if (force ||
+      ticket_manager.get_front_ticket() != ticket_manager.get_back_ticket()) {
+    auto [previous_front, current_front] = ticket_manager.pop_front_ticket();
+    // if pop was successful - front changed, notify waiting threads
+    if (force || previous_front != current_front) {
+      MUTEX_LOCK(guard, &this->m_lock_wait_for_ticket_turn);
+      mysql_cond_broadcast(&this->m_cond_wait_for_ticket_turn);
+    }
+  }
+}
+
+void Commit_stage_manager::update_session_ticket_state(THD *thd) {
+  auto &ticket_ctx = thd->rpl_thd_ctx.binlog_group_commit_ctx();
+  if (ticket_ctx.has_waited()) return;
+  if (ticket_ctx.get_session_ticket() >
+      binlog::Bgc_ticket_manager::instance().get_coalesced_ticket())
+    this->update_ticket_manager(1, ticket_ctx.get_session_ticket());
+  ticket_ctx.mark_as_already_waited();
+}
+
+void Commit_stage_manager::update_ticket_manager(
+    std::uint64_t sessions_count, const binlog::BgcTicket &session_ticket) {
+  auto &ticket_manager = binlog::Bgc_ticket_manager::instance();
+  ticket_manager.add_processed_sessions_to_front_ticket(sessions_count,
+                                                        session_ticket);
+
+  DBUG_EXECUTE_IF("rpl_end_of_ticket_blocked", {
+    const char act[] =
+        "now signal signal.end_of_ticket_waiting wait_for "
+        "signal.end_of_ticket_continue";
+    assert(!debug_sync_set_action(current_thd, STRING_WITH_LEN(act)));
+  });
+
+  this->signal_end_of_ticket();
+}
+
+void Commit_stage_manager::finish_session_ticket(THD *thd) {
+  auto &ticket_ctx = thd->rpl_thd_ctx.binlog_group_commit_ctx();
+  if (ticket_ctx.get_session_ticket().is_set()) {
+    this->wait_for_ticket_turn(thd);
+    ticket_ctx.reset();
+  }
+}
+
+void Commit_stage_manager::disable_manual_session_tickets() {
+  if (!Binlog_group_commit_ctx::manual_ticket_setting()->load()) return;
+  Binlog_group_commit_ctx::manual_ticket_setting()->store(false);
+  binlog::Bgc_ticket_manager::instance().coalesce();
+  Commit_stage_manager::get_instance().signal_end_of_ticket(true);
+}
+
+void Commit_stage_manager::enable_manual_session_tickets() {
+  Binlog_group_commit_ctx::manual_ticket_setting()->store(true);
+}
+
 #ifndef NDEBUG
 void Commit_stage_manager::clear_preempt_status(THD *head) {
   assert(head);
