@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -98,6 +99,17 @@ Gtid_set::Gtid_set(Sid_map *_sid_map, const char *text,
   *status = add_gtid_text(text);
 }
 
+void Gtid_set::claim_memory_ownership(bool claim) {
+  m_intervals.claim_memory_ownership(claim);
+
+  Interval_chunk *chunk = chunks;
+  while (chunk != nullptr) {
+    Interval_chunk *next_chunk = chunk->next;
+    my_claim(chunk, claim);
+    chunk = next_chunk;
+  }
+}
+
 void Gtid_set::init() {
   DBUG_TRACE;
   has_cached_string_length = false;
