@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,19 +72,20 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 */
 static const size_t ring_buffer_size = 5 * 1024 * 1024;
 
-static char *ring_buffer_start = nullptr;  //< buffer start
-static char *ring_buffer_end = nullptr;    //< buffer end (for convenience)
+static char *ring_buffer_start = nullptr;  ///< buffer start
+static char *ring_buffer_end = nullptr;    ///< buffer end (for convenience)
 
-static char *ring_buffer_write = nullptr;  //< write position ("head")
-static char *ring_buffer_read = nullptr;   //< read pos (oldest entry, "tail")
+static char *ring_buffer_write = nullptr;  ///< write position ("head")
+static char *ring_buffer_read = nullptr;   ///< read pos (oldest entry, "tail")
 
-ulong log_sink_pfs_buffered_bytes = 0;   //< bytes in use (now)
-ulong log_sink_pfs_buffered_events = 0;  //< events in buffer (now)
-ulong log_sink_pfs_expired_events = 0;   //< number of expired entries (ever)
-ulong log_sink_pfs_longest_event = 0;    //< longest event seen (ever)
-ulonglong log_sink_pfs_latest_timestamp = 0;  //< timestamp of most recent write
+ulong log_sink_pfs_buffered_bytes = 0;   ///< bytes in use (now)
+ulong log_sink_pfs_buffered_events = 0;  ///< events in buffer (now)
+ulong log_sink_pfs_expired_events = 0;   ///< number of expired entries (ever)
+ulong log_sink_pfs_longest_event = 0;    ///< longest event seen (ever)
+ulonglong log_sink_pfs_latest_timestamp =
+    0;  ///< timestamp of most recent write
 
-PSI_memory_key key_memory_log_sink_pfs;  //< memory instrumentation
+PSI_memory_key key_memory_log_sink_pfs;  ///< memory instrumentation
 
 /**
   ring-buffer rwlock
@@ -809,7 +811,7 @@ int log_error_read_log_init() {
 
   @retval          int                  number of added fields, if any
 */
-int log_sink_perfschema(void *instance MY_ATTRIBUTE((unused)), log_line *ll) {
+int log_sink_perfschema(void *instance [[maybe_unused]], log_line *ll) {
   log_sink_pfs_event e;
   memset(&e, 0, sizeof(log_sink_pfs_event));
 
