@@ -1,15 +1,16 @@
-/* Copyright (c) 2017, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,8 +43,8 @@
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_const.h"
 #include "sql/sql_plugin.h"
-#include "sql/strfunc.h"          // find_type
-#include "sql/sys_vars_shared.h"  // intern_find_sys_var
+#include "sql/strfunc.h"  // find_type
+#include "sql/sys_vars_shared.h"
 #include "sql/system_variables.h"
 #include "sql_string.h"
 #include "template_utils.h"
@@ -304,7 +305,7 @@ TYPELIB *sys_var_pluginvar::plugin_var_typelib(void) {
 }
 
 uchar *sys_var_pluginvar::do_value_ptr(THD *running_thd, THD *target_thd,
-                                       enum_var_type type, LEX_STRING *) {
+                                       enum_var_type type, std::string_view) {
   uchar *result;
 
   result = real_value_ptr(target_thd, type);
@@ -419,12 +420,14 @@ bool sys_var_pluginvar::global_update(THD *thd, set_var *var) {
   }
 
   if ((plugin_var->flags & PLUGIN_VAR_TYPEMASK) == PLUGIN_VAR_STR &&
-      plugin_var->flags & PLUGIN_VAR_MEMALLOC)
+      plugin_var->flags & PLUGIN_VAR_MEMALLOC) {
     rc = plugin_var_memalloc_global_update(thd, plugin_var,
                                            static_cast<char **>(tgt),
                                            *static_cast<char *const *>(src));
-  else
+  } else {
     plugin_var->update(thd, plugin_var, tgt, src);
+    return (thd->is_error() ? 1 : 0);
+  }
 
   return rc;
 }
@@ -497,7 +500,7 @@ ulonglong sys_var_pluginvar::get_max_value() {
   @retval true not valid
   @retval false valid
 */
-bool sys_var_pluginvar::on_check_pluginvar(sys_var *self MY_ATTRIBUTE((unused)),
+bool sys_var_pluginvar::on_check_pluginvar(sys_var *self [[maybe_unused]],
                                            THD *, set_var *var) {
   /* This handler is installed only if NO_DEFAULT is specified */
   assert(((sys_var_pluginvar *)self)->plugin_var->flags & PLUGIN_VAR_NODEFAULT);
@@ -604,10 +607,10 @@ int check_func_bool(THD *, SYS_VAR *, void *save, st_mysql_value *value) {
       goto err;
   } else {
     if (value->val_int(value, &tmp) < 0) goto err;
-    if (tmp > 1) goto err;
+    if (tmp > 1 || tmp < 0) goto err;
     result = (int)tmp;
   }
-  *(bool *)save = result ? true : false;
+  *(bool *)save = (result != 0);
   return 0;
 err:
   return 1;
@@ -814,10 +817,8 @@ st_bookmark *find_bookmark(const char *plugin, const char *name, int flags) {
   varname[0] = flags & PLUGIN_VAR_TYPEMASK;
 
   const auto it = get_bookmark_hash()->find(std::string(varname, length - 1));
-  if (it == get_bookmark_hash()->end())
-    return nullptr;
-  else
-    return it->second;
+  if (it == get_bookmark_hash()->end()) return nullptr;
+  return it->second;
 }
 
 void plugin_opt_set_limits(struct my_option *options, const SYS_VAR *opt) {
