@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -127,7 +128,7 @@ bool str_to_time(const CHARSET_INFO *cs, const char *str, size_t length,
                  MYSQL_TIME *l_time, my_time_flags_t flags,
                  MYSQL_TIME_STATUS *status) {
   char cnv[MAX_TIME_FULL_WIDTH + 3];  // +3 for nanoseconds (for rounding)
-  if ((cs->state & MY_CS_NONASCII) != 0) {
+  if (!my_charset_is_ascii_based(cs)) {
     length = to_ascii(cs, str, length, cnv, sizeof(cnv));
     str = cnv;
   }
@@ -148,7 +149,7 @@ bool str_to_datetime(const CHARSET_INFO *cs, const char *str, size_t length,
                      MYSQL_TIME *l_time, my_time_flags_t flags,
                      MYSQL_TIME_STATUS *status) {
   char cnv[MAX_DATETIME_FULL_WIDTH + 3];  // +3 for nanoseconds (for rounding)
-  if ((cs->state & MY_CS_NONASCII) != 0) {
+  if (!my_charset_is_ascii_based(cs)) {
     length = to_ascii(cs, str, length, cnv, sizeof(cnv));
     str = cnv;
   }
@@ -183,7 +184,9 @@ bool str_to_datetime_with_warn(String *str, MYSQL_TIME *l_time,
                                      ErrConvString(str), l_time->time_type,
                                      NullS))
       return true;
+    if (ret_val) status.squelch_deprecation();
   }
+  check_deprecated_datetime_format(current_thd, str->charset(), status);
 
   if (ret_val) return true;
   return convert_time_zone_displacement(thd->time_zone(), l_time);
@@ -208,7 +211,8 @@ static bool lldiv_t_to_datetime(lldiv_t lld, MYSQL_TIME *ltime,
     if (!*warnings) /* Neither sets warnings in case of ZERO DATE */
       *warnings |= MYSQL_TIME_WARN_TRUNCATED;
     return true;
-  } else if (ltime->time_type == MYSQL_TIMESTAMP_DATE) {
+  }
+  if (ltime->time_type == MYSQL_TIMESTAMP_DATE) {
     /*
       Generate a warning in case of DATE with fractional part:
         20011231.1234 -> '2001-12-31'
@@ -224,6 +228,25 @@ static bool lldiv_t_to_datetime(lldiv_t lld, MYSQL_TIME *ltime,
   return false;
 }
 
+/**
+  Convert decimal value to datetime.
+
+  @param       decimal The value to convert from.
+  @param[out]  ltime   The variable to convert to.
+  @param       flags   Conversion flags.
+
+  @returns false on success, true if not convertible to datetime.
+*/
+bool decimal_to_datetime(const my_decimal *decimal, MYSQL_TIME *ltime,
+                         my_time_flags_t flags) {
+  lldiv_t lld;
+  if (my_decimal2lldiv_t(0, decimal, &lld)) {
+    return true;
+  }
+  int warnings = 0;
+  return lldiv_t_to_datetime(lld, ltime, flags, &warnings);
+}
+
 /**
   Convert decimal value to datetime value with a warning.
   @param       decimal The value to convert from.
@@ -256,6 +279,24 @@ bool my_decimal_to_datetime_with_warn(const my_decimal *decimal,
   return rc;
 }
 
+/**
+  Convert double value to datetime.
+
+  @param       nr      The value to convert from.
+  @param[out]  ltime   The variable to convert to.
+  @param       flags   Conversion flags.
+
+  @returns false on success, true if not convertible to datetime.
+*/
+bool double_to_datetime(double nr, MYSQL_TIME *ltime, my_time_flags_t flags) {
+  lldiv_t lld;
+  if (double2lldiv_t(nr, &lld)) {
+    return true;
+  }
+  int warnings = 0;
+  return lldiv_t_to_datetime(lld, ltime, flags, &warnings);
+}
+
 /**
   Convert double value to datetime value with a warning.
   @param       nr      The value to convert from.
@@ -331,6 +372,24 @@ static bool lldiv_t_to_time(lldiv_t lld, MYSQL_TIME *ltime, int *warnings) {
                                           current_thd->is_fsp_truncate_mode());
 }
 
+/**
+  Convert decimal number to TIME
+
+  @param      decimal  The number to convert from.
+  @param[out] ltime          The variable to convert to.
+
+  @returns false on success, true if not convertible to time.
+*/
+bool decimal_to_time(const my_decimal *decimal, MYSQL_TIME *ltime) {
+  lldiv_t lld;
+  int warnings = 0;
+
+  if (my_decimal2lldiv_t(0, decimal, &lld)) {
+    return true;
+  }
+  return lldiv_t_to_time(lld, ltime, &warnings);
+}
+
 /**
   Convert decimal number to TIME
   @param      decimal  The number to convert from.
@@ -360,6 +419,23 @@ bool my_decimal_to_time_with_warn(const my_decimal *decimal,
   return rc;
 }
 
+/**
+  Convert double number to TIME
+
+  @param      nr      The number to convert from.
+  @param[out] ltime   The variable to convert to.
+
+  @returns false on success, true if not convertible to time.
+*/
+bool double_to_time(double nr, MYSQL_TIME *ltime) {
+  lldiv_t lld;
+  if (double2lldiv_t(nr, &lld) != E_DEC_OK) {
+    return true;
+  }
+  int warnings = 0;
+  return lldiv_t_to_time(lld, ltime, &warnings);
+}
+
 /**
   Convert double number to TIME
 
@@ -437,8 +513,7 @@ bool my_longlong_to_time_with_warn(longlong nr, MYSQL_TIME *ltime) {
 */
 bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
                                               const Time_zone &tz,
-                                              struct timeval *tm,
-                                              int *warnings) {
+                                              my_timeval *tm, int *warnings) {
   if (!ltime->month) /* Zero date */
   {
     assert(!ltime->year && !ltime->day);
@@ -450,19 +525,20 @@ bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
       *warnings |= MYSQL_TIME_WARN_TRUNCATED;
       return true;
     }
-    tm->tv_sec = tm->tv_usec = 0;  // '0000-00-00 00:00:00.000000'
+    tm->m_tv_sec = tm->m_tv_usec = 0;  // '0000-00-00 00:00:00.000000'
     return false;
   }
 
   bool is_in_dst_time_gap = false;
-  if (!(tm->tv_sec = tz.TIME_to_gmt_sec(ltime, &is_in_dst_time_gap))) {
+  if (!(tm->m_tv_sec = tz.TIME_to_gmt_sec(ltime, &is_in_dst_time_gap))) {
     /*
       Date was outside of the supported timestamp range.
       For example: '3001-01-01 00:00:00' or '1000-01-01 00:00:00'
     */
     *warnings |= MYSQL_TIME_WARN_OUT_OF_RANGE;
     return true;
-  } else if (is_in_dst_time_gap) {
+  }
+  if (is_in_dst_time_gap) {
     /*
       Set MYSQL_TIME_WARN_INVALID_TIMESTAMP warning to indicate
       that date was fine but pointed to winter/summer time switch gap.
@@ -471,7 +547,7 @@ bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
     */
     *warnings |= MYSQL_TIME_WARN_INVALID_TIMESTAMP;
   }
-  tm->tv_usec = ltime->second_part;
+  tm->m_tv_usec = ltime->second_part;
   return false;
 }
 
@@ -502,7 +578,7 @@ bool datetime_with_no_zero_in_date_to_timeval(const MYSQL_TIME *ltime,
   @return False on success, true on error.
 */
 bool datetime_to_timeval(const MYSQL_TIME *ltime, const Time_zone &tz,
-                         struct timeval *tm, int *warnings) {
+                         my_timeval *tm, int *warnings) {
   return check_date(*ltime, non_zero_date(*ltime), TIME_NO_ZERO_IN_DATE,
                     warnings) ||
          datetime_with_no_zero_in_date_to_timeval(ltime, tz, tm, warnings);
@@ -531,7 +607,7 @@ bool str_to_time_with_warn(String *str, MYSQL_TIME *l_time) {
                                      NullS))
       return true;
   }
-
+  check_deprecated_datetime_format(current_thd, str->charset(), status);
   if (!ret_val)
     if (convert_time_zone_displacement(thd->time_zone(), l_time)) return true;
 
@@ -581,7 +657,7 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
    @page DEFAULT_TIME_FUNCS Functions to create default time/date/datetime
    strings
    @note
-    For the moment the Date_time_format argument is ignored becasue
+    For the moment the Date_time_format argument is ignored because
     MySQL doesn't support comparing of date/time/datetime strings that
     are not in arbutary order as dates are compared as strings in some
     context)
@@ -599,7 +675,7 @@ const char *get_date_time_format_str(const Known_date_time_format *format,
   @param[out] str      String to convert to
   @param      dec      Number of fractional digits.
 */
-void make_time(const Date_time_format *format MY_ATTRIBUTE((unused)),
+void make_time(const Date_time_format *format [[maybe_unused]],
                const MYSQL_TIME *l_time, String *str, uint dec) {
   uint length = static_cast<uint>(my_time_to_str(*l_time, str->ptr(), dec));
   str->length(length);
@@ -612,7 +688,7 @@ void make_time(const Date_time_format *format MY_ATTRIBUTE((unused)),
   @param      l_time   DATE value
   @param[out] str      String to convert to
 */
-void make_date(const Date_time_format *format MY_ATTRIBUTE((unused)),
+void make_date(const Date_time_format *format [[maybe_unused]],
                const MYSQL_TIME *l_time, String *str) {
   uint length = static_cast<uint>(my_date_to_str(*l_time, str->ptr()));
   str->length(length);
@@ -626,7 +702,7 @@ void make_date(const Date_time_format *format MY_ATTRIBUTE((unused)),
   @param[out] str      String to convert to
   @param      dec      Number of fractional digits.
 */
-void make_datetime(const Date_time_format *format MY_ATTRIBUTE((unused)),
+void make_datetime(const Date_time_format *format [[maybe_unused]],
                    const MYSQL_TIME *l_time, String *str, uint dec) {
   uint length = static_cast<uint>(my_datetime_to_str(*l_time, str->ptr(), dec));
   str->length(length);
