@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,10 +25,9 @@
 
 #include "sql/sql_udf.h"
 
-#include "my_config.h"
-
 #include <stdio.h>
 #include <string.h>
+
 #include <iterator>
 #include <memory>
 #include <new>
@@ -40,6 +40,7 @@
 #include "map_helpers.h"
 #include "my_alloc.h"
 #include "my_base.h"
+#include "my_config.h"
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_io.h"
@@ -50,30 +51,31 @@
 #include "my_sys.h"
 #include "my_thread_local.h"
 #include "mysql/components/service_implementation.h"
+#include "mysql/components/services/bits/mysql_rwlock_bits.h"
 #include "mysql/components/services/bits/psi_bits.h"
+#include "mysql/components/services/bits/psi_memory_bits.h"
+#include "mysql/components/services/bits/psi_rwlock_bits.h"
 #include "mysql/components/services/log_builtins.h"
 #include "mysql/components/services/log_shared.h"
-#include "mysql/components/services/mysql_rwlock_bits.h"
-#include "mysql/components/services/psi_memory_bits.h"
-#include "mysql/components/services/psi_rwlock_bits.h"
 #include "mysql/psi/mysql_memory.h"
 #include "mysql/psi/mysql_rwlock.h"
 #include "mysql_com.h"
 #include "mysqld_error.h"  // ER_*
+#include "sql/derror.h"    // ER_THD
 #include "sql/field.h"
 #include "sql/handler.h"
+#include "sql/iterators/row_iterator.h"
 #include "sql/mdl.h"
-#include "sql/mysqld.h"   // opt_allow_suspicious_udfs
-#include "sql/records.h"  // unique_ptr_destroy_only<RowIterator>
-#include "sql/row_iterator.h"
+#include "sql/mysqld.h"     // opt_allow_suspicious_udfs
 #include "sql/sql_base.h"   // close_mysql_tables
 #include "sql/sql_class.h"  // THD
 #include "sql/sql_const.h"
+#include "sql/sql_executor.h"            // unique_ptr_destroy_only<RowIterator>
 #include "sql/sql_parse.h"               // check_string_char_length
 #include "sql/sql_plugin.h"              // check_valid_path
 #include "sql/sql_system_table_check.h"  // System_table_intact
 #include "sql/sql_table.h"               // write_bin_log
-#include "sql/table.h"                   // TABLE_LIST
+#include "sql/table.h"                   // Table_ref
 #include "sql/thd_raii.h"
 #include "sql/thr_malloc.h"
 #include "sql/transaction.h"  // trans_*
@@ -106,6 +108,8 @@ static MEM_ROOT mem;
 static collation_unordered_map<std::string, udf_func *> *udf_hash;
 static mysql_rwlock_t THR_LOCK_udf;
 
+static constexpr const size_t UDF_ALLOC_BLOCK_SIZE{1024};
+
 static udf_func *add_udf(LEX_STRING *name, Item_result ret, char *dl,
                          Item_udftype typ);
 static void udf_hash_delete(udf_func *udf);
@@ -122,7 +126,7 @@ static const TABLE_FIELD_TYPE
         {{STRING_WITH_LEN("dl")}, {STRING_WITH_LEN("char(128)")}, {nullptr, 0}},
         {{STRING_WITH_LEN("type")},
          {STRING_WITH_LEN("enum('function','aggregate')")},
-         {STRING_WITH_LEN("utf8")}}};
+         {STRING_WITH_LEN("utf8mb3")}}};
 static const TABLE_FIELD_DEF mysql_udf_table_def = {MYSQL_UDF_TABLE_FIELD_COUNT,
                                                     mysql_udf_table_fields};
 
@@ -187,7 +191,7 @@ static void init_udf_psi_keys(void) {
 /**
   Initialize the UDF global structures.
   This is done as a separate step so that the UDF registration
-  service can work when initalizing plugins, which happens
+  service can work when initializing plugins, which happens
   before reading the UDF table.
 */
 void udf_init_globals() {
@@ -199,7 +203,7 @@ void udf_init_globals() {
 #endif
 
   mysql_rwlock_init(key_rwlock_THR_LOCK_udf, &THR_LOCK_udf);
-  init_sql_alloc(key_memory_udf_mem, &mem, UDF_ALLOC_BLOCK_SIZE, 0);
+  init_sql_alloc(key_memory_udf_mem, &mem, UDF_ALLOC_BLOCK_SIZE);
 
   udf_hash = new collation_unordered_map<std::string, udf_func *>(
       system_charset_info, key_memory_udf_mem);
@@ -228,7 +232,7 @@ void udf_read_functions_table() {
   THD *new_thd = new (std::nothrow) THD;
   if (new_thd == nullptr) {
     LogErr(ERROR_LEVEL, ER_UDF_CANT_ALLOC_FOR_STRUCTURES);
-    free_root(&mem, MYF(0));
+    mem.Clear();
     delete new_thd;
     return;
   }
@@ -239,7 +243,7 @@ void udf_read_functions_table() {
     new_thd->set_db(db_lex_cstr);
   }
 
-  TABLE_LIST tables(db, "func", TL_READ, MDL_SHARED_READ_ONLY);
+  Table_ref tables(db, "func", TL_READ, MDL_SHARED_READ_ONLY);
 
   if (open_trans_system_tables_for_read(new_thd, &tables)) {
     DBUG_PRINT("error", ("Can't open udf table"));
@@ -248,9 +252,9 @@ void udf_read_functions_table() {
   }
 
   table = tables.table;
-  iterator = init_table_iterator(new_thd, table, nullptr,
-                                 /*ignore_not_found_rows=*/false,
-                                 /*count_examined_rows=*/false);
+  iterator =
+      init_table_iterator(new_thd, table, /*ignore_not_found_rows=*/false,
+                          /*count_examined_rows=*/false);
   if (iterator == nullptr) goto end;
   while (!(error = iterator->Read())) {
     DBUG_PRINT("info", ("init udf record"));
@@ -258,7 +262,7 @@ void udf_read_functions_table() {
     name.str = get_field(&mem, table->field[0]);
 
     // Check the name.str is NULL or not.
-    if (name.str == NULL) {
+    if (name.str == nullptr) {
       sql_print_error("Invalid row in mysql.func table for column 'name'");
       continue;
     }
@@ -266,7 +270,7 @@ void udf_read_functions_table() {
     name.length = strlen(name.str);
     char *dl_name = get_field(&mem, table->field[2]);
 
-    if (dl_name == NULL) {
+    if (dl_name == nullptr) {
       sql_print_error("Invalid row in mysql.func table for function '%.64s'",
                       name.str);
       continue;
@@ -312,7 +316,7 @@ void udf_read_functions_table() {
         // Print warning to log
         LogErr(ERROR_LEVEL, ER_FAILED_TO_OPEN_SHARED_LIBRARY, tmp->dl,
                error_number, errmsg);
-        // Keep the udf in the hash so that we can remove it later
+        udf_hash_delete(tmp);
         continue;
       }
       new_dl = true;
@@ -332,14 +336,15 @@ void udf_read_functions_table() {
   table->invalidate_dict();  // Force close to free memory
 
 end:
-  close_trans_system_tables(new_thd);
+  if (new_thd->is_attachable_ro_transaction_active())
+    close_trans_system_tables(new_thd);
   delete new_thd;
 }
 
 /**
-   Deintialize the UDF subsystem.
+   Deinitialize the UDF subsystem.
 
-   This function closes the shared libaries.
+   This function closes the shared libraries.
 */
 void udf_unload_udfs() {
   DBUG_TRACE;
@@ -361,7 +366,7 @@ void udf_unload_udfs() {
 }
 
 /**
-   Deintialize the UDF subsystem.
+   Deinitialize the UDF subsystem.
 
    This function does the following:
    1. Free the UDF hash.
@@ -374,7 +379,7 @@ void udf_deinit_globals() {
     delete udf_hash;
     udf_hash = nullptr;
   }
-  free_root(&mem, MYF(0));
+  mem.Clear();
   initialized = false;
 
   mysql_rwlock_destroy(&THR_LOCK_udf);
@@ -407,7 +412,7 @@ static void udf_hash_delete(udf_func *udf) {
   } else {
     /*
       The functions is in use ; Rename the functions instead of removing it.
-      The functions will be automaticly removed when the least threads
+      The functions will be automatically removed when the least threads
       doesn't use it anymore
     */
     udf_hash->erase(it);
@@ -468,7 +473,7 @@ static void *find_udf_dl(const char *dl) {
 
   if (!dl) return nullptr;
   /*
-    Because only the function name is hashed, we have to search trough
+    Because only the function name is hashed, we have to search through
     all rows to find the dl.
   */
   for (const auto &key_and_value : *udf_hash) {
@@ -594,13 +599,14 @@ static bool udf_end_transaction(THD *thd, bool rollback, udf_func *udf,
 
   @param thd                 THD context.
   @param udf                 Pointer to UDF function.
+  @param if_not_exists       True if 'IF NOT EXISTS' clause was specified.
 
   @note Like implementations of other DDL/DML in MySQL, this function
   relies on the caller to close the thread tables. This is done in the
   end of dispatch_command().
 */
 
-bool mysql_create_function(THD *thd, udf_func *udf) {
+bool mysql_create_function(THD *thd, udf_func *udf, bool if_not_exists) {
   bool error = true;
   void *dl = nullptr;
   int new_dl = 0;
@@ -640,7 +646,7 @@ bool mysql_create_function(THD *thd, udf_func *udf) {
     Acquire MDL SNRW for TL_WRITE type so that deadlock and
     timeout errors are avoided from the Storage Engine.
   */
-  TABLE_LIST tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
+  Table_ref tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
 
   if (open_and_lock_tables(thd, &tables, MYSQL_LOCK_IGNORE_TIMEOUT))
     return error;
@@ -673,8 +679,19 @@ bool mysql_create_function(THD *thd, udf_func *udf) {
 
   mysql_rwlock_rdlock(&THR_LOCK_udf);
   if (udf_hash->count(to_string(udf->name)) != 0) {
-    my_error(ER_UDF_EXISTS, MYF(0), udf->name.str);
     mysql_rwlock_unlock(&THR_LOCK_udf);
+
+    // UDF with the same name already exists
+    if (if_not_exists) {
+      push_warning_printf(thd, Sql_condition::SL_NOTE, ER_UDF_EXISTS,
+                          ER_THD(thd, ER_UDF_EXISTS), udf->name.str);
+      error = (write_bin_log(thd, true, thd->query().str, thd->query().length,
+                             false) != 0);
+      if (error) error = udf_end_transaction(thd, error, nullptr, false);
+
+    } else {
+      my_error(ER_UDF_EXISTS, MYF(0), udf->name.str);
+    }
     return error;
   }
   dl = find_udf_dl(udf->dl);
@@ -766,7 +783,7 @@ bool mysql_drop_function(THD *thd, const LEX_STRING *udf_name) {
     return error;
   }
 
-  TABLE_LIST tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
+  Table_ref tables("mysql", "func", TL_WRITE, MDL_SHARED_NO_READ_WRITE);
 
   if (open_and_lock_tables(thd, &tables, MYSQL_LOCK_IGNORE_TIMEOUT))
     return error;
