@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 2017, 2023, Oracle and/or its affiliates.
+Copyright (c) 2017, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -231,9 +232,8 @@ int Arch_Group::mark_active() {
 
   ut_ad(m_active_file.m_file == OS_FILE_CLOSED);
 
-  m_active_file =
-      os_file_create(innodb_arch_file_key, m_active_file_name, option,
-                     OS_FILE_NORMAL, OS_CLONE_LOG_FILE, false, &success);
+  m_active_file = os_file_create(innodb_arch_file_key, m_active_file_name,
+                                 option, OS_CLONE_LOG_FILE, false, &success);
 
   int err = (success ? 0 : ER_CANT_OPEN_FILE);
 
@@ -267,9 +267,8 @@ int Arch_Group::mark_durable() {
 
   ut_ad(m_durable_file.m_file == OS_FILE_CLOSED);
 
-  m_durable_file =
-      os_file_create(innodb_arch_file_key, m_durable_file_name, option,
-                     OS_FILE_NORMAL, OS_CLONE_LOG_FILE, false, &success);
+  m_durable_file = os_file_create(innodb_arch_file_key, m_durable_file_name,
+                                  option, OS_CLONE_LOG_FILE, false, &success);
 
   int err = (success ? 0 : ER_CANT_OPEN_FILE);
 
@@ -750,7 +749,7 @@ bool Arch_File_Ctx::validate(Arch_Group *group, uint file_index,
   pfs_os_file_t file;
 
   file = os_file_create(innodb_arch_file_key, file_name, OS_FILE_OPEN,
-                        OS_FILE_NORMAL, OS_CLONE_LOG_FILE, true, &success);
+                        OS_CLONE_LOG_FILE, true, &success);
 
   if (!success) {
     return (false);
@@ -1227,15 +1226,6 @@ uint64_t Arch_Block::get_file_offset(uint64_t block_num, Arch_Blk_Type type) {
   return offset;
 }
 
-bool Arch_Block::is_zeroes(const byte *block) {
-  for (ulint i = 0; i < ARCH_PAGE_BLK_SIZE; i++) {
-    if (block[i] != 0) {
-      return (false);
-    }
-  }
-  return (true);
-}
-
 bool Arch_Block::validate(byte *block) {
   auto data_length = Arch_Block::get_data_len(block);
   auto block_checksum = Arch_Block::get_checksum(block);
@@ -1246,7 +1236,7 @@ bool Arch_Block::validate(byte *block) {
         << Arch_Block::get_block_number(block);
     ut_d(ut_error);
     ut_o(return (false));
-  } else if (Arch_Block::is_zeroes(block)) {
+  } else if (ut::is_zeros(block, ARCH_PAGE_BLK_SIZE)) {
     return (false);
   }
 
@@ -1728,7 +1718,7 @@ void Arch_Page_Sys::track_page(buf_page_t *bpage, lsn_t track_lsn,
       m_state = ARCH_STATE_ABORT;
       arch_oper_mutex_exit();
       ut_d(ut_error);
-      ut_o(return );
+      ut_o(return);
     }
 
     cur_blk = m_data.get_block(&m_write_pos, ARCH_DATA_BLOCK);
@@ -2933,7 +2923,7 @@ int Arch_Group::read_from_file(Arch_Page_Pos *read_pos, uint read_len,
   /* Open file in read only mode. */
   pfs_os_file_t file =
       os_file_create(innodb_arch_file_key, file_name, OS_FILE_OPEN,
-                     OS_FILE_NORMAL, OS_CLONE_LOG_FILE, true, &success);
+                     OS_CLONE_LOG_FILE, true, &success);
 
   if (!success) {
     my_error(ER_CANT_OPEN_FILE, MYF(0), file_name, errno,
