@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2021, Oracle and/or its affiliates.
+Copyright (c) 1994, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -32,6 +33,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include <sys/types.h>
 #include <new>
+#include <type_traits>
 
 #include "data0data.h"
 #include "ha_prototypes.h"
@@ -64,8 +66,8 @@ bool dtuple_coll_eq(const dtuple_t *tuple1, const dtuple_t *tuple2) {
 
   ut_ad(tuple1 != nullptr);
   ut_ad(tuple2 != nullptr);
-  ut_ad(tuple1->magic_n == DATA_TUPLE_MAGIC_N);
-  ut_ad(tuple2->magic_n == DATA_TUPLE_MAGIC_N);
+  ut_ad(tuple1->magic_n == dtuple_t::MAGIC_N);
+  ut_ad(tuple2->magic_n == dtuple_t::MAGIC_N);
   ut_ad(dtuple_check_typed(tuple1));
   ut_ad(dtuple_check_typed(tuple2));
 
@@ -146,7 +148,7 @@ static bool dtuple_check_typed_no_assert(const dtuple_t *tuple) {
 bool dfield_check_typed(const dfield_t *field) {
   if (dfield_get_type(field)->mtype > DATA_MTYPE_CURRENT_MAX ||
       dfield_get_type(field)->mtype < DATA_MTYPE_CURRENT_MIN) {
-    ib::fatal(ER_IB_MSG_158)
+    ib::fatal(UT_LOCATION_HERE, ER_IB_MSG_158)
         << "Data field type " << dfield_get_type(field)->mtype << ", len "
         << dfield_get_len(field);
   }
@@ -168,7 +170,7 @@ bool dtuple_check_typed(const dtuple_t *tuple) {
 }
 
 bool dtuple_validate(const dtuple_t *tuple) {
-  ut_ad(tuple->magic_n == DATA_TUPLE_MAGIC_N);
+  ut_ad(tuple->magic_n == dtuple_t::MAGIC_N);
 
   auto n_fields = dtuple_get_n_fields(tuple);
 
@@ -180,7 +182,7 @@ bool dtuple_validate(const dtuple_t *tuple) {
     auto len = dfield_get_len(field);
 
     if (!dfield_is_null(field)) {
-      const byte *data;
+      const byte *data [[maybe_unused]];
 
       data = static_cast<const byte *>(dfield_get_data(field));
 #ifndef UNIV_DEBUG_VALGRIND
@@ -330,7 +332,7 @@ void dfield_print_also_hex(const dfield_t *dfield) {
       }
 
       data = static_cast<byte *>(dfield_get_data(dfield));
-      /* fall through */
+      [[fallthrough]];
 
     case DATA_BINARY:
     default:
@@ -353,7 +355,7 @@ void dfield_print_also_hex(const dfield_t *dfield) {
 static void dfield_print_raw(FILE *f, const dfield_t *dfield) {
   ulint len = dfield_get_len(dfield);
   if (!dfield_is_null(dfield)) {
-    ulint print_len = ut_min(len, static_cast<ulint>(1000));
+    ulint print_len = std::min(len, static_cast<ulint>(1000));
     ut_print_buf(f, dfield_get_data(dfield), print_len);
     if (len != print_len) {
       fprintf(f, "(total %lu bytes%s)", (ulong)len,
@@ -427,7 +429,7 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
   dfield_t *dfield;
   dict_field_t *ifield;
   ulint size;
-  ulint n_fields;
+  ulint n_fields [[maybe_unused]];
   ulint local_len;
   ulint local_prefix_len;
 
@@ -455,7 +457,8 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
   }
 
   heap = mem_heap_create(
-      size + dtuple_get_n_fields(entry) * sizeof(big_rec_field_t) + 1000);
+      size + dtuple_get_n_fields(entry) * sizeof(big_rec_field_t) + 1000,
+      UT_LOCATION_HERE);
 
   vector = big_rec_t::alloc(heap, dtuple_get_n_fields(entry));
 
@@ -480,6 +483,8 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
       dfield = dtuple_get_nth_field(entry, i);
       ifield = index->get_field(i);
 
+      ut_ad(dfield_get_len(dfield) != UNIV_SQL_INSTANT_DROP_COL);
+
       /* Skip fixed-length, NULL, externally stored,
       or short columns */
 
@@ -564,12 +569,12 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
     }
 
 #if 0
-		/* The following would fail the Valgrind checks in
-		page_cur_insert_rec_low() and page_cur_insert_rec_zip().
-		The BLOB pointers in the record will be initialized after
-		the record and the BLOBs have been written. */
-		UNIV_MEM_ALLOC(data + local_prefix_len,
-			       BTR_EXTERN_FIELD_REF_SIZE);
+                /* The following would fail the Valgrind checks in
+                page_cur_insert_rec_low() and page_cur_insert_rec_zip().
+                The BLOB pointers in the record will be initialized after
+                the record and the BLOBs have been written. */
+                UNIV_MEM_ALLOC(data + local_prefix_len,
+                               BTR_EXTERN_FIELD_REF_SIZE);
 #endif
 
     dfield_set_data(dfield, data, local_len);
@@ -583,6 +588,8 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
 
       upd_field_t upd_field;
       upd_field.field_no = longest_i;
+      IF_DEBUG(upd_field.field_phy_pos =
+                   index->get_field(longest_i)->col->get_col_phy_pos();)
       upd_field.orig_len = 0;
       upd_field.exp = nullptr;
       upd_field.old_v_val = nullptr;
@@ -788,9 +795,8 @@ trx_id_t dtuple_t::get_trx_id() const {
 }
 
 void dtuple_t::ignore_trailing_default(const dict_index_t *index) {
-  if (!index->has_instant_cols()) {
-    return;
-  }
+  ut_a(index->has_instant_cols());
+  ut_a(!index->has_row_versions());
 
   /* It's necessary to check all the fields that could be default.
   If it's from normal update, it should be OK to keep original
@@ -800,6 +806,11 @@ void dtuple_t::ignore_trailing_default(const dict_index_t *index) {
   it has to check all possible default values. */
   for (; n_fields > index->get_instant_fields(); --n_fields) {
     const dict_col_t *col = index->get_field(n_fields - 1)->col;
+
+    /* We shall never come here if INSTANT ADD/DROP is done in a version. */
+    ut_a(!col->is_instant_added());
+    ut_a(!col->is_instant_dropped());
+
     const dfield_t *dfield = dtuple_get_nth_field(this, n_fields - 1);
     ulint len = dfield_get_len(dfield);
 
@@ -872,12 +883,21 @@ void multi_value_data::alloc(uint32_t num, bool alc_bitset, mem_heap_t *heap) {
 void multi_value_data::alloc_bitset(mem_heap_t *heap, uint32_t size) {
   ut_ad(bitset == nullptr);
 
-  bitset = static_cast<Bitset *>(mem_heap_zalloc(heap, sizeof(Bitset)));
-  uint32_t alloc_size = (size == 0 ? num_v : size);
-  byte *bitmap =
-      static_cast<byte *>(mem_heap_zalloc(heap, UT_BITS_IN_BYTES(alloc_size)));
-  bitset->init(bitmap, UT_BITS_IN_BYTES(alloc_size));
+  bitset = static_cast<Bitset<> *>(mem_heap_zalloc(heap, sizeof(Bitset<>)));
+  /*TODO: why is it num_v? Shouldn't it be num_alc? Looks like copy_low assumes
+  that this->bitset points to a bitmap which is at least as large as other's
+  multi_value->bitset->size(), as it is copying bytes from it, yet the only
+  assert protection is that num_alc is larger than that. So, what's preventing
+  num_alc > multi_value->bitset->size() > num_v = bitmap_bytes? */
+  uint32_t alloc_size = UT_BITS_IN_BYTES(size == 0 ? num_v : size);
+  byte *bitmap = static_cast<byte *>(mem_heap_zalloc(heap, alloc_size));
+  ut_d(bitmap_bytes = alloc_size);
+  new (bitset) Bitset(bitmap, alloc_size);
   bitset->set();
+  static_assert(
+      std::is_trivially_destructible_v<Bitset<>>,
+      "Bitset<> must be trivially destructible in order to be able to "
+      "(de)allocate it via mem_heap_t");
 }
 
 uint32_t Multi_value_logger::get_log_len(bool precise) const {
@@ -942,12 +962,17 @@ byte *Multi_value_logger::log(byte **ptr) {
     ut_memcpy(*ptr, m_mv_data->datap[i], m_mv_data->data_len[i]);
     *ptr += m_mv_data->data_len[i];
   }
-
+  /* This fragment is optional, which will be detected by reader by comparing
+  the amount of data read, to the length we will write to *old_ptr later. */
   if (m_mv_data->bitset != nullptr) {
     /* Always just write out the bitset of enough size for all data,
     rather than the size of bitset. */
     uint32_t bitset_len = UT_BITS_IN_BYTES(m_mv_data->num_v);
-    ut_memcpy(*ptr, m_mv_data->bitset->bitset(), bitset_len);
+    /* make sure we will not read past the buffer */
+    ut_ad(bitset_len <= m_mv_data->bitmap_bytes);
+    /* make sure we will not miss actual data */
+    ut_ad(m_mv_data->bitset->size_bytes() <= bitset_len);
+    ut_memcpy(*ptr, m_mv_data->bitset->data(), bitset_len);
     *ptr += bitset_len;
   }
 
@@ -1007,7 +1032,10 @@ const byte *Multi_value_logger::read(const byte *ptr, dfield_t *field,
 
   if (ptr < old_ptr + total_len) {
     multi_val->alloc_bitset(heap);
-    multi_val->bitset->copy(ptr, UT_BITS_IN_BYTES(num));
+    ut_ad(num == multi_val->num_v);
+    ut_ad(UT_BITS_IN_BYTES(num) == multi_val->bitmap_bytes);
+    ut_a(UT_BITS_IN_BYTES(num) == multi_val->bitset->size_bytes());
+    multi_val->bitset->copy_from(ptr);
     ptr += UT_BITS_IN_BYTES(num);
   }
 
