@@ -1,18 +1,19 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2023, Oracle and/or its affiliates.
+Copyright (c) 1994, 2024, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -560,6 +561,12 @@ dberr_t btr_sdi_create_index(space_id_t space_id, bool dict_locked);
 constexpr uint32_t BTR_N_LEAF_PAGES = 1;
 constexpr uint32_t BTR_TOTAL_SIZE = 2;
 
+/** Check if the given index is empty.  An index is considered empty if it
+has only the root page with no user records, including del-marked records.
+@param[in]   index   index
+@return true if index is empty, false otherwise. */
+bool btr_is_index_empty(const dict_index_t *index);
+
 #include "btr0btr.ic"
 
 #endif
