@@ -1,17 +1,18 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2021, Oracle and/or its affiliates.
+Copyright (c) 1996, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -44,8 +45,6 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include <set>
 #include <vector>
 
-//#include <unordered_set>
-
 /** printf(3) format used for printing DB_TRX_ID and other system fields */
 #define TRX_ID_FMT IB_ID_FMT
 
@@ -53,25 +52,20 @@ this program; if not, write to the Free Software Foundation, Inc.,
 static const space_id_t TRX_SYS_SPACE = 0;
 
 /** Page number of the transaction system page */
-#define TRX_SYS_PAGE_NO FSP_TRX_SYS_PAGE_NO
+constexpr uint32_t TRX_SYS_PAGE_NO = FSP_TRX_SYS_PAGE_NO;
 
 /** Random value to check for corruption of trx_t */
 static const ulint TRX_MAGIC_N = 91118598;
 
 /** If this flag is set then the transaction cannot be rolled back
 asynchronously. */
-static const ib_uint32_t TRX_FORCE_ROLLBACK_DISABLE = 1 << 29;
-
-/** Was the transaction rolled back asynchronously or by the
-owning thread. This flag is relevant only if TRX_FORCE_ROLLBACK
-is set.  */
-static const ib_uint32_t TRX_FORCE_ROLLBACK_ASYNC = 1 << 30;
+static const uint32_t TRX_FORCE_ROLLBACK_DISABLE = 1 << 29;
 
 /** Mark the transaction for forced rollback */
-static const ib_uint32_t TRX_FORCE_ROLLBACK = 1 << 31;
+static const uint32_t TRX_FORCE_ROLLBACK = 1U << 31;
 
-/** For masking out the above four flags */
-static const ib_uint32_t TRX_FORCE_ROLLBACK_MASK = 0x1FFFFFFF;
+/** For masking out the above flags */
+static const uint32_t TRX_FORCE_ROLLBACK_MASK = 0x1FFFFFFF;
 
 /** Transaction execution states when trx->state == TRX_STATE_ACTIVE */
 enum trx_que_t {
@@ -148,7 +142,7 @@ typedef ib_id_t roll_ptr_t;
 typedef ib_id_t undo_no_t;
 
 /** Maximum transaction identifier */
-#define TRX_ID_MAX IB_ID_MAX
+constexpr trx_id_t TRX_ID_MAX = IB_ID_MAX;
 
 /** Transaction savepoint */
 struct trx_savept_t {
@@ -179,6 +173,43 @@ typedef ib_mutex_t UndoMutex;
 typedef ib_mutex_t PQMutex;
 typedef ib_mutex_t TrxSysMutex;
 
+/** Used to identify trx uniquely over time */
+struct trx_guid_t {
+  /** The immutable id of trx_t object - if you have a pointer to trx_t then we
+  guarantee that immutable id of it will not change over time. Also there are
+  never two trx_t objects at the same time with same immutable id. However it
+  may happen that two different transactions that do not occur at the same time
+  reuse the same trx_t and thus have same immutable id. Use m_version to detect
+  this situation. */
+  uint64_t m_immutable_id{};
+
+  /** As trx_t objects and thus immutable ids can be reused we need also trx's
+  version, which is incremented each time trx_t object gets reused. */
+  uint64_t m_version{};
+
+  /** Initializes trx_guid_t object to a value which doesn't match any real
+  transaction. */
+  trx_guid_t() = default;
+
+  /** Initializes trx_guid_t with data uniquely identifying the transaction
+  represented by trx_t object.
+  @param[in]  trx   the object representing the transaction */
+  trx_guid_t(const trx_t &trx);
+
+  /** Checks if two guids represent the same transaction:
+  they refer to the same trx_t struct and it was not reused meanwhile.
+  @param[in]  rhs   another guid to compare against
+  @return true iff the two guids are equal and thus represent same transaction*/
+  bool operator==(const trx_guid_t &rhs) const {
+    return m_immutable_id == rhs.m_immutable_id && m_version == rhs.m_version;
+  }
+
+  /** Checks if the instance is non-empty, i.e. was not default-constructed,
+  but rather initialized to correspond to a real trx_t.
+  @return true iff this guid was initialized to match a real transaction */
+  operator bool() const { return m_immutable_id != 0; }
+};
+
 /** The rollback segment memory object */
 struct trx_rseg_t {
 #ifdef UNIV_DEBUG
@@ -246,21 +277,22 @@ struct trx_rseg_t {
   page_no_t curr_size{};
 
  public:
+  using Undo_list = UT_LIST_BASE_NODE_T_EXTERN(trx_undo_t, undo_list);
   /*--------------------------------------------------------*/
   /* Fields for update undo logs */
   /** List of update undo logs */
-  UT_LIST_BASE_NODE_T(trx_undo_t) update_undo_list;
+  Undo_list update_undo_list;
 
   /** List of update undo log segments cached for fast reuse */
-  UT_LIST_BASE_NODE_T(trx_undo_t) update_undo_cached;
+  Undo_list update_undo_cached;
 
   /*--------------------------------------------------------*/
   /* Fields for insert undo logs */
   /** List of insert undo logs */
-  UT_LIST_BASE_NODE_T(trx_undo_t) insert_undo_list;
+  Undo_list insert_undo_list;
 
   /** List of insert undo log segments cached for fast reuse */
-  UT_LIST_BASE_NODE_T(trx_undo_t) insert_undo_cached;
+  Undo_list insert_undo_cached;
 
   /*--------------------------------------------------------*/
 
@@ -292,7 +324,7 @@ inline std::ostream &operator<<(std::ostream &out, const trx_rseg_t &rseg) {
   return (rseg.print(out));
 }
 
-using Rsegs_Vector = std::vector<trx_rseg_t *, ut_allocator<trx_rseg_t *>>;
+using Rsegs_Vector = std::vector<trx_rseg_t *, ut::allocator<trx_rseg_t *>>;
 using Rseg_Iterator = Rsegs_Vector::iterator;
 
 /** This is a wrapper for a std::vector of trx_rseg_t object pointers. */
@@ -322,7 +354,7 @@ class Rsegs {
   void clear();
 
   /** Add rollback segment.
-  @param[in]	rseg	rollback segment to add. */
+  @param[in]    rseg    rollback segment to add. */
   void push_back(trx_rseg_t *rseg) { m_rsegs.push_back(rseg); }
 
   /** Number of registered rsegs.
@@ -338,12 +370,12 @@ class Rsegs {
   Rseg_Iterator end() { return (m_rsegs.end()); }
 
   /** Find the rseg at the given slot in this vector.
-  @param[in]	slot	a slot within the vector.
+  @param[in]    slot    a slot within the vector.
   @return an iterator to the end */
   trx_rseg_t *at(ulint slot) { return (m_rsegs.at(slot)); }
 
   /** Find an rseg in the std::vector that uses the rseg_id given.
-  @param[in]	rseg_id		A slot in a durable array such as
+  @param[in]    rseg_id         A slot in a durable array such as
                                   the TRX_SYS page or RSEG_ARRAY page.
   @return a pointer to an trx_rseg_t that uses the rseg_id. */
   trx_rseg_t *find(ulint rseg_id);
@@ -359,16 +391,16 @@ class Rsegs {
         [](trx_rseg_t *lhs, trx_rseg_t *rhs) { return (rhs->id > lhs->id); });
   }
 
-  /** Get a shared lock on m_rsegs. */
-  void s_lock() { rw_lock_s_lock(m_latch); }
+  /** Acquire the shared lock on m_rsegs. */
+  void s_lock() { rw_lock_s_lock(m_latch, UT_LOCATION_HERE); }
 
-  /** Get a shared lock on m_rsegs. */
+  /** Release the shared lock on m_rsegs. */
   void s_unlock() { rw_lock_s_unlock(m_latch); }
 
-  /** Get a shared lock on m_rsegs. */
-  void x_lock() { rw_lock_x_lock(m_latch); }
+  /** Acquire the exclusive lock on m_rsegs. */
+  void x_lock() { rw_lock_x_lock(m_latch, UT_LOCATION_HERE); }
 
-  /** Get a shared lock on m_rsegs. */
+  /** Release the exclusive lock on m_rsegs. */
   void x_unlock() { rw_lock_x_unlock(m_latch); }
 
   /** Return whether the undo tablespace is active.
@@ -438,12 +470,12 @@ class Rsegs {
   ACTIVE:   The rollback segments in this tablespace can be allocated to new
             transactions.  The undo tablespace is ready for undo logs.
   INACTIVE_IMPLICIT: These rollback segments are no longer being used by new
-            transactions.  They arre 'inactive'. The truncate process
+            transactions.  They are 'inactive'. The truncate process
             is happening. This undo tablespace was selected by the
             purge thread implicitly. When the truncation process
             is complete, the next state is ACTIVE.
   INACTIVE_EXPLICIT:  These rollback segments are no longer being used by new
-            transactions.  They arre 'inactive'. The truncate process
+            transactions.  They are 'inactive'. The truncate process
             is happening. This undo tablespace was selected by the
             an ALTER UNDO TABLESPACE  SET INACTIVE command. When the
             truncation process is complete, the next state is EMPTY.
@@ -462,7 +494,7 @@ class Rsegs {
   SetInactive:  This ALTER UNDO TABLESPACE causes an explicit truncation.
   SetActive:    This ALTER UNDO TABLESPACE changes the target state from
                 EMPTY to ACTIVE.
-  Trucate:      The truncate process is completed by the purge thread.
+  Truncate:     The truncate process is completed by the purge thread.
   Drop:         Delete an EMPTY undo tablespace
   Crash:        A crash occurs
   Fixup:        At startup, if an undo space was being truncated with a crash.
@@ -514,48 +546,61 @@ class Rsegs {
   undo_space_states m_state;
 };
 
-/** Rollback segements from a given transaction with trx-no
+template <size_t N>
+using Rsegs_array = std::array<trx_rseg_t *, N>;
+
+/** Rollback segments from a given transaction with trx-no
 scheduled for purge. */
 class TrxUndoRsegs {
  public:
-  /** Default constructor */
-  TrxUndoRsegs() : m_trx_no() {}
-
   explicit TrxUndoRsegs(trx_id_t trx_no) : m_trx_no(trx_no) {
-    // Do nothing
+    for (auto &rseg : m_rsegs) {
+      rseg = nullptr;
+    }
   }
 
+  /** Default constructor */
+  TrxUndoRsegs() : TrxUndoRsegs(0) {}
+
+  void set_trx_no(trx_id_t trx_no) { m_trx_no = trx_no; }
+
   /** Get transaction number
   @return trx_id_t - get transaction number. */
   trx_id_t get_trx_no() const { return (m_trx_no); }
 
   /** Add rollback segment.
   @param rseg rollback segment to add. */
-  void push_back(trx_rseg_t *rseg) { m_rsegs.push_back(rseg); }
-
-  /** Erase the element pointed by given iterator.
-  @param[in]	it	iterator */
-  void erase(Rseg_Iterator &it) { m_rsegs.erase(it); }
+  void insert(trx_rseg_t *rseg) {
+    for (size_t i = 0; i < m_rsegs_n; ++i) {
+      if (m_rsegs[i] == rseg) {
+        return;
+      }
+    }
+    ut_a(m_rsegs_n < 2);
+    m_rsegs[m_rsegs_n++] = rseg;
+  }
 
   /** Number of registered rsegs.
   @return size of rseg list. */
-  ulint size() const { return (m_rsegs.size()); }
+  size_t size() const { return (m_rsegs_n); }
 
   /**
   @return an iterator to the first element */
-  Rseg_Iterator begin() { return (m_rsegs.begin()); }
+  typename Rsegs_array<2>::iterator begin() { return m_rsegs.begin(); }
 
   /**
   @return an iterator to the end */
-  Rseg_Iterator end() { return (m_rsegs.end()); }
+  typename Rsegs_array<2>::iterator end() {
+    return m_rsegs.begin() + m_rsegs_n;
+  }
 
   /** Append rollback segments from referred instance to current
   instance. */
-  void append(const TrxUndoRsegs &append_from) {
+  void insert(const TrxUndoRsegs &append_from) {
     ut_ad(get_trx_no() == append_from.get_trx_no());
-
-    m_rsegs.insert(m_rsegs.end(), append_from.m_rsegs.begin(),
-                   append_from.m_rsegs.end());
+    for (size_t i = 0; i < append_from.m_rsegs_n; ++i) {
+      insert(append_from.m_rsegs[i]);
+    }
   }
 
   /** Compare two TrxUndoRsegs based on trx_no.
@@ -574,57 +619,25 @@ class TrxUndoRsegs {
   /** The rollback segments transaction number. */
   trx_id_t m_trx_no;
 
+  size_t m_rsegs_n{};
+
   /** Rollback segments of a transaction, scheduled for purge. */
-  Rsegs_Vector m_rsegs;
+  Rsegs_array<2> m_rsegs;
 };
 
 typedef std::priority_queue<
-    TrxUndoRsegs, std::vector<TrxUndoRsegs, ut_allocator<TrxUndoRsegs>>,
+    TrxUndoRsegs, std::vector<TrxUndoRsegs, ut::allocator<TrxUndoRsegs>>,
     TrxUndoRsegs>
     purge_pq_t;
 
-typedef std::vector<trx_id_t, ut_allocator<trx_id_t>> trx_ids_t;
-
-/** Mapping read-write transactions from id to transaction instance, for
-creating read views and during trx id lookup for MVCC and locking. */
-struct TrxTrack {
-  explicit TrxTrack(trx_id_t id, trx_t *trx = nullptr) : m_id(id), m_trx(trx) {
-    // Do nothing
-  }
-
-  trx_id_t m_id;
-  trx_t *m_trx;
-};
-
-struct TrxTrackHash {
-  size_t operator()(const TrxTrack &key) const { return (size_t(key.m_id)); }
-};
-
-/**
-Comparator for TrxMap */
-struct TrxTrackHashCmp {
-  bool operator()(const TrxTrack &lhs, const TrxTrack &rhs) const {
-    return (lhs.m_id == rhs.m_id);
-  }
-};
-
-/**
-Comparator for TrxMap */
-struct TrxTrackCmp {
-  bool operator()(const TrxTrack &lhs, const TrxTrack &rhs) const {
-    return (lhs.m_id < rhs.m_id);
-  }
-};
-
-// typedef std::unordered_set<TrxTrack, TrxTrackHash, TrxTrackHashCmp> TrxIdSet;
-typedef std::set<TrxTrack, TrxTrackCmp, ut_allocator<TrxTrack>> TrxIdSet;
+typedef std::vector<trx_id_t, ut::allocator<trx_id_t>> trx_ids_t;
 
 struct TrxVersion {
   TrxVersion(trx_t *trx);
 
   trx_t *m_trx;
-  ulint m_version;
+  uint64_t m_version;
 };
 
-typedef std::vector<TrxVersion, ut_allocator<TrxVersion>> hit_list_t;
+typedef std::vector<TrxVersion, ut::allocator<TrxVersion>> hit_list_t;
 #endif /* trx0types_h */
