@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -93,12 +94,17 @@ class PFS_buffer_default_array {
     size_t monotonic_max;
     value_type *pfs;
 
-    if (m_full) {
+    if (m_full.load()) {
       return nullptr;
     }
 
+    // Immutable
+    size_t max = m_max.load();
+    // Immutable
+    T *ptr = m_ptr.load();
+
     monotonic = m_monotonic.m_size_t++;
-    monotonic_max = monotonic + m_max;
+    monotonic_max = monotonic + max;
 
     if (unlikely(monotonic >= monotonic_max)) {
       /*
@@ -108,12 +114,12 @@ class PFS_buffer_default_array {
       */
       m_monotonic.m_size_t.store(0);
       monotonic = 0;
-      monotonic_max = m_max;
+      monotonic_max = max;
     }
 
     while (monotonic < monotonic_max) {
-      index = monotonic % m_max;
-      pfs = m_ptr + index;
+      index = monotonic % max;
+      pfs = ptr + index;
 
       if (pfs->m_lock.free_to_dirty(dirty_state)) {
         return pfs;
@@ -121,25 +127,68 @@ class PFS_buffer_default_array {
       monotonic = m_monotonic.m_size_t++;
     }
 
-    m_full = true;
+    m_full.store(true);
     return nullptr;
   }
 
   void deallocate(value_type *pfs) {
     pfs->m_lock.allocated_to_free();
-    m_full = false;
+
+    if (m_full.load()) {
+      m_full.store(false);
+    }
   }
 
   T *get_first() { return m_ptr; }
 
   T *get_last() { return m_ptr + m_max; }
 
-  bool m_full;
+  /**
+   Page full flag.
+
+   Concurrency profile:
+   - mostly read during normal operations.
+   - do not perform useless write,
+     check for previous value first
+   - occasional write on state changes
+  */
+  std::atomic<bool> m_full{false};
+
+  /**
+   Monotonic counter.
+   This counter is used to access items in the page.
+
+   Concurrency profile:
+   - frequent read and write, must be on its own cacheline
+  */
   PFS_cacheline_atomic_size_t m_monotonic;
-  T *m_ptr;
-  size_t m_max;
-  /** Container. */
-  PFS_opaque_container *m_container;
+
+  /**
+   Array of values.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<T *> m_ptr{nullptr};
+
+  /**
+   Max number of items in the page.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_max{0};
+
+  /**
+   Container.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<PFS_opaque_container *> m_container{nullptr};
 };
 
 template <class T>
@@ -444,12 +493,14 @@ class PFS_buffer_scalable_container {
       return;
     }
 
+    allocator_type *allocator = m_allocator.load();
+
     native_mutex_lock(&m_critical_section);
 
     for (i = 0; i < PFS_PAGE_COUNT; i++) {
       page = m_pages[i];
       if (page != nullptr) {
-        m_allocator->free_array(page);
+        allocator->free_array(page);
         delete page;
         m_pages[i] = nullptr;
       }
@@ -477,7 +528,7 @@ class PFS_buffer_scalable_container {
   size_t get_memory() { return get_row_count() * get_row_size(); }
 
   value_type *allocate(pfs_dirty_state *dirty_state) {
-    if (m_full) {
+    if (m_full.load()) {
       m_lost++;
       return nullptr;
     }
@@ -599,9 +650,12 @@ class PFS_buffer_scalable_container {
           builtin_memory_scalable_buffer.count_alloc(sizeof(array_type));
 
           array->m_max = get_page_logical_size(current_page_count);
-          int rc = m_allocator->alloc_array(array);
+
+          allocator_type *allocator = m_allocator.load();
+
+          int rc = allocator->alloc_array(array);
           if (rc != 0) {
-            m_allocator->free_array(array);
+            allocator->free_array(array);
             delete array;
             builtin_memory_scalable_buffer.count_free(sizeof(array_type));
             m_lost++;
@@ -638,7 +692,7 @@ class PFS_buffer_scalable_container {
     }
 
     m_lost++;
-    m_full = true;
+    m_full.store(true);
     return nullptr;
   }
 
@@ -651,10 +705,14 @@ class PFS_buffer_scalable_container {
     safe_pfs->m_lock.allocated_to_free();
 
     /* Flag the containing page as not full. */
-    page->m_full = false;
+    if (page->m_full.load()) {
+      page->m_full.store(false);
+    }
 
     /* Flag the overall container as not full. */
-    m_full = false;
+    if (m_full.load()) {
+      m_full.store(false);
+    }
   }
 
   static void static_deallocate(value_type *safe_pfs) {
@@ -666,7 +724,9 @@ class PFS_buffer_scalable_container {
     safe_pfs->m_lock.allocated_to_free();
 
     /* Flag the containing page as not full. */
-    page->m_full = false;
+    if (page->m_full.load()) {
+      page->m_full.store(false);
+    }
 
     /* Find the containing buffer */
     PFS_opaque_container *opaque_container = page->m_container;
@@ -674,7 +734,9 @@ class PFS_buffer_scalable_container {
     container = reinterpret_cast<container_type *>(opaque_container);
 
     /* Flag the overall container as not full. */
-    container->m_full = false;
+    if (container->m_full.load()) {
+      container->m_full.store(false);
+    }
   }
 
   iterator_type iterate() {
@@ -902,15 +964,89 @@ class PFS_buffer_scalable_container {
     return nullptr;
   }
 
-  bool m_initialized;
-  bool m_full;
-  size_t m_max;
+  /**
+   Initialized full flag.
+
+   Concurrency profile:
+   - write in init / cleanup
+   - readonly during normal operations
+  */
+  std::atomic<bool> m_initialized{false};
+
+  /**
+   Buffer full flag.
+
+   Concurrency profile:
+   - mostly read during normal operations.
+   - do not perform useless write,
+     check for previous value first
+   - occasional write on state changes
+  */
+  std::atomic<bool> m_full{false};
+
+  /**
+   Max number of items in the buffer.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_max{0};
+
+  /**
+   Monotonic page counter.
+   This counter is used to access pages in the array.
+
+   Concurrency profile:
+   - frequent read and write, must be on its own cacheline
+  */
   PFS_cacheline_atomic_size_t m_monotonic;
+
+  /**
+   Current page index.
+
+   Concurrency profile:
+   - occasional write on buffer extend
+   - mostly read otherwise
+  */
   PFS_cacheline_atomic_size_t m_max_page_index;
-  size_t m_max_page_count;
-  size_t m_last_page_size;
+
+  /**
+   Max number of pages.
+
+   Concurrency profile:
+   - written once on buffer creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_max_page_count{0};
+
+  /**
+   Size of the last page.
+
+   Concurrency profile:
+   - written once on buffer creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_last_page_size{0};
+
+  /**
+   Array of pages.
+
+   Concurrency profile:
+   - occasional write on buffer extend
+   - mostly read otherwise
+  */
   std::atomic<array_type *> m_pages[PFS_PAGE_COUNT];
-  allocator_type *m_allocator;
+
+  /**
+   Buffer allocator.
+
+   Concurrency profile:
+   - written once on buffer creation
+   - read only during normal operations.
+  */
+  std::atomic<allocator_type *> m_allocator{nullptr};
+
   native_mutex_t m_critical_section;
 };
 
@@ -968,7 +1104,7 @@ class PFS_buffer_scalable_iterator {
 template <class T>
 class PFS_buffer_processor {
  public:
-  virtual ~PFS_buffer_processor<T>() = default;
+  virtual ~PFS_buffer_processor() = default;
   virtual void operator()(T *element) = 0;
 };
 
