@@ -1,30 +1,30 @@
-/* Copyright (c) 2007, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2007, 2024, Oracle and/or its affiliates.
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License, version 2.0, as published by the Free Software Foundation.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
 
-   This library is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the library and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   permission to link the program and your derivative works with the
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
    Universal FOSS Exception, version 1.0, a copy of which can be found at
    http://oss.oracle.com/licenses/universal-foss-exception.
 
-   This library is distributed in the hope that it will be useful,
+   This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License, version 2.0, for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License, version 2.0, for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-   MA 02110-1301  USA */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 /****************************************************************
 
@@ -328,7 +328,7 @@ size_t my_gcvt(double x, my_gcvt_arg_type type, int width, char *to,
 
   /*
     Number of digits in the exponent from the 'e' conversion.
-     The sign of the exponent is taken into account separetely, we don't need
+     The sign of the exponent is taken into account separately, we don't need
      to count it here.
    */
   exp_len = 1 + (decpt >= 101 || decpt <= -99) + (decpt >= 11 || decpt <= -9);
@@ -342,9 +342,9 @@ size_t my_gcvt(double x, my_gcvt_arg_type type, int width, char *to,
      2. 0 < decpt < len, i.e. we have "NNN.NNN" => F = len + 1
      3. len <= decpt, i.e. we have "NNN00" => F = decpt
   */
-  have_space =
-      (decpt <= 0 ? len - decpt + 2
-                  : decpt > 0 && decpt < len ? len + 1 : decpt) <= width;
+  have_space = (decpt <= 0                 ? len - decpt + 2
+                : decpt > 0 && decpt < len ? len + 1
+                                           : decpt) <= width;
   /*
     The following is true when no significant digits can be placed with the
     specified field width using the 'f' format, and the 'e' format
@@ -355,7 +355,7 @@ size_t my_gcvt(double x, my_gcvt_arg_type type, int width, char *to,
     Assume that we don't have enough space to place all significant digits in
     the 'f' format. We have to choose between the 'e' format and the 'f' one
     to keep as many significant digits as possible.
-    Let E and F be the lengths of decimal representaion in the 'e' and 'f'
+    Let E and F be the lengths of decimal representation in the 'e' and 'f'
     formats, respectively. We want to use the 'f' format if, and only if F <= E.
     Consider the following cases:
     1. decpt <= 0.
@@ -832,8 +832,7 @@ static Bigint *s2b(const char *s, int nd0, int nd, ULong y9,
   i = 9;
   if (9 < nd0) {
     s += 9;
-    do
-      b = multadd(b, 10, *s++ - '0', alloc);
+    do b = multadd(b, 10, *s++ - '0', alloc);
     while (++i < nd0);
     s++; /* skip '.' */
   } else
@@ -1053,8 +1052,7 @@ static Bigint *lshift(Bigint *b, int k, Stack_alloc *alloc) {
     } while (x < xe);
     if ((*x1 = z)) ++n1;
   } else
-    do
-      *x1++ = *x++;
+    do *x1++ = *x++;
     while (x < xe);
   b1->wds = n1 - 1;
   Bfree(b, alloc);
@@ -1295,7 +1293,7 @@ static double my_strtod_int(const char *s00, const char **se, int *error,
   for (s = s00; s < end; s++) switch (*s) {
       case '-':
         sign = 1;
-        // Fall through.
+        [[fallthrough]];
       case '+':
         s++;
         goto break2;
@@ -1312,6 +1310,7 @@ static double my_strtod_int(const char *s00, const char **se, int *error,
 break2:
   if (s >= end) goto ret0;
 
+  // Gobble up leading zeros.
   if (*s == '0') {
     nz0 = 1;
     while (++s < end && *s == '0')
@@ -1328,6 +1327,7 @@ break2:
   nd0 = nd;
   if (s < end && c == '.') {
     if (++s < end) c = *s;
+    // Only leading zeros, now count number of leading zeros after the '.'
     if (!nd) {
       for (; s < end; ++s) {
         c = *s;
@@ -1344,6 +1344,13 @@ break2:
     for (; s < end; ++s) {
       c = *s;
       if (c < '0' || c > '9') break;
+
+      // We have seen some digits, but not enough of them are non-zero.
+      // Gobble up all the rest of the digits, and look for exponent.
+      if (nd > 0 && nz > DBL_MAX_10_EXP) {
+        continue;
+      }
+
       /*
         Here we are parsing the fractional part.
         We can stop counting digits after a while: the extra digits
@@ -1377,12 +1384,13 @@ dig_done:
     if (++s < end) switch (c = *s) {
         case '-':
           esign = 1;
-          // Fall through.
+          [[fallthrough]];
         case '+':
           if (++s < end) c = *s;
       }
     if (s < end && c >= '0' && c <= '9') {
-      while (s < end && c == '0') c = *++s;
+      while (s < end && *s == '0') ++s;  // Skip leading zeros in exponent.
+      if (s < end) c = *s;  // First significant digit in exponent, if any.
       if (s < end && c > '0' && c <= '9') {
         L = c - '0';
         s1 = s;
@@ -2087,14 +2095,14 @@ static char *dtoa(double dd, int mode, int ndigits, int *decpt, int *sign,
       break;
     case 2:
       leftright = 0;
-      // Fall through.
+      [[fallthrough]];
     case 4:
       if (ndigits <= 0) ndigits = 1;
       ilim = ilim1 = i = ndigits;
       break;
     case 3:
       leftright = 0;
-      // Fall through.
+      [[fallthrough]];
     case 5:
       i = ndigits + k + 1;
       ilim = i;
